package bran.spring.jacn;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.DOMOutputter;
import org.jdom.xpath.XPath;
import org.objectweb.asm.ClassReader;
import org.springframework.core.io.ClassPathResource;
import org.springframework.transaction.interceptor.TransactionProxyFactoryBean;
import org.w3c.dom.Document;

public class ConvertUtil {
	/**
	 * convert from 'Lcom/.../;' to com.mycomapny...
	 * @param desc
	 * @return
	 */
	public static String desc2reg(String desc) {
		if (desc.startsWith("L")) {
			desc = desc.substring(1);
		}
		
		if (desc.endsWith(";")) {
			desc = desc.substring(0, desc.length() - 1);
		}
		
		return desc.replace('/', '.');
	}

	/**
	 * 
	 * @param className fully qualified Java class name
	 * @param id bean id
	 * @return
	 */
	public static  Element newBeanDef(String className, String id) {
		Element e = new Element("bean");
		e.getAttributes().add(new org.jdom.Attribute("id", id));
		e.getAttributes().add(new org.jdom.Attribute("class", className));
		return e;
	}

	static Element newValueProp(String propName, String value) {
		Element r = newPropElement(propName);
		r.getChildren().add(newValueElement(value));
		return r;
	}

	static  Element newValueElement(String value) {
		Element e = new Element("value");
		e.setText(value);
		return e;
	}

	static  Element newPropRefNode(String propName, String beanId) {
		Element r = newPropElement(propName);
		r.getChildren().add(newRefBean(beanId));
		return r;
	}

	static  Element newPropElement(String propName) {
		Element r = new Element("property");
		r.getAttributes().add(new Attribute("name", propName));
		return r;
	}

	static Element newRefBean(String beanId) {
		Element e = new Element("ref");
		e.getAttributes().add(new Attribute("bean", beanId));
		return e;
	}

	/**
	 * get the property name from
	 * @param setterName
	 * @return
	 */
	public static String getPropFromMethodName(String setterName) {
		if (setterName.startsWith("set") 
				&& setterName.length() > 3 
				&& Character.isUpperCase(setterName.charAt(3))) {
			String rest = "";
			if (setterName.length() > 4)
				rest = setterName.substring(4);
			if (Character.isUpperCase(rest.charAt(0))) {
				return setterName.substring(3);
			}
			else {
				char c = setterName.substring(3, 4).charAt(0);
				c = Character.toLowerCase(c);
				return String.valueOf(c) + rest;
			}
		}
		else {
			throw new RuntimeException("bad method name for extracting property");
		}
	}

	static String getDescType(String desc) {
		if (desc.equals("Z"))
			return "boolean";
		if (desc.equals("B"))
			return "byte";
		if (desc.equals("C"))
			return "char";
		if (desc.equals("D"))
			return "double";
		if (desc.equals("F"))
			return "float";
		if (desc.equals("I"))
			return "int";
		if (desc.equals("J"))
			return "long";
		if (desc.equals("S"))
			return "short";
		if (desc.equals("V"))
			return "void";
		return ConvertUtil.getType(desc);
	}

	static String getType(String internalName) {
		if (internalName.startsWith("[")){
			// support 1-dimmensional array for now
			internalName  = internalName.substring(1);
		}
		if (internalName.endsWith(";")){
			internalName = internalName.substring(0, internalName.length() - 1);
			if (internalName.startsWith("L")){
				internalName  = internalName.substring(1);
			}					
		}
		
		return internalName.replace('/', '.');
	}

	static public String jacn2Xml(InputStream is) throws IOException {
		org.jdom.Document doc = getJdomDoc(is);
		return JacnClassVisitor.toXml(doc);
	}

	/**
	 * conver to JDom Document
	 * @param is
	 * @return
	 * @throws IOException
	 */
	public static org.jdom.Document getJdomDoc(InputStream is) throws IOException {
		ClassReader cr;
/*		
		// let's copy the inputstream. We'll be using it twice
		ArrayList al = new ArrayList(10000);

		//		 todo: seems not i18n compatible
		int b = is.read();
		while (b != -1) {
			al.add(new Byte((byte) b)); 
			b = is.read();
		}
		byte[] ba = new byte[al.size()];
		for (int i = 0; i < ba.length; i++) {
			ba[i] = ((Byte)al.get(i)).byteValue();
		} 

		ByteArrayInputStream bais = new ByteArrayInputStream(ba);
		is.close();

		cr = new ClassReader(bais);
		
		ClassNode cn = new ClassNode();
		cr.accept(cn, false);
		
		bais.reset();
*/		cr = new ClassReader(is);
		JacnClassVisitor visitor = new JacnClassVisitor(); 
		cr.accept(visitor, false);
		is.close();
		return visitor.getXmlDoc();
	}
	
	static public String jacn2Xml(Class cls) throws IOException {
		String cc = cls.getName().replace('.', '/') + ".class";
		return jacn2Xml(new ClassPathResource(cc).getInputStream());
	}
	
	/**
	 * convert jacn config class to the w3c Document
	 * @param cls
	 * @return
	 * @throws IOException
	 * @throws JDOMException
	 */
	static public Document jacn2Dom(Class cls) throws IOException, JDOMException {
		String cc = cls.getName().replace('.', '/') + ".class";
		org.jdom.Document doc = getJdomDoc(new ClassPathResource(cc).getInputStream());
		DOMOutputter dommer = new DOMOutputter();
		return dommer.output(doc);
	}
	/**
	 * convert jacn config class to the JDom Document
	 * @param cls
	 * @return
	 * @throws IOException
	 * @throws JDOMException
	 */
	static public org.jdom.Document jacn2JDom(Class cls) throws IOException, JDOMException {
		String cc = cls.getName().replace('.', '/') + ".class";
		return getJdomDoc(new ClassPathResource(cc).getInputStream());
	}
	
	/**
	 * try to dump as much as possible from the source XML to Jacn class format. 
	 * Lots of manual post=processing is required. 
	 * @return
	 * @throws IOException 
	 * @throws JDOMException 
	 */
	static public String[] xml2Jacn(File xmlDocument) throws JDOMException, IOException {
		StringBuffer fields = new StringBuffer();
		StringBuffer imports = new StringBuffer();
		StringBuffer wire = new StringBuffer();
		
		List topBeans =new ArrayList();
		Map replacers = new HashMap();
		
		imports.append("import " + AbstractJacnConfigFile.class.getName() + ";\r\n");
		
		SAXBuilder saxBuilder = new SAXBuilder();
		org.jdom.Document jdomDocument = saxBuilder.build(xmlDocument);
		List beans = jdomDocument.getRootElement().getChildren();
		//	lets build the top level bean name list
		for (Iterator iter = beans.iterator(); iter.hasNext();) {
			Element bean = (Element) iter.next();
			topBeans.add(bean.getAttributeValue("id"));
		}	
		
//)(XPath.selectSingleNode(jdomDocument, "")
		for (Iterator iter = beans.iterator(); iter.hasNext();) {
			wire.append('\n');
			
			Element bean = (Element) iter.next();
			Attribute attId = bean.getAttribute("id");
			String beanId = attId.getValue();
			Attribute attClass = bean.getAttribute("class");
			String clz = null;
			String fullClassName = null;
			if (attClass != null) {
				clz = attClass.getValue();
				fullClassName = clz;
				if (TransactionProxyFactoryBean.class.getName().equals(fullClassName)) {
					//;
				}
				else {
					imports.append("import " + clz + ";\n");
					clz = clz.substring(clz.lastIndexOf('.') + 1);
				}
			}

			Attribute attSingleton = bean.getAttribute("singleton");
			String singleton = "static";
			if (attSingleton != null) {
				singleton = attSingleton .getValue();
				if ("false".equals(singleton)) {
					singleton = "";
				}
				else {
					singleton = "static";
				}
			}
			
			Attribute attLazy = bean.getAttribute("lazy-init");
			if (attLazy != null) {
				String lazy = attLazy.getValue();
				if ("true".equals(lazy)) {
					wire.append("\t\tlazyInit(" + beanId + ");\n");
				}
			}
			
			Attribute attAutowire = bean.getAttribute("autowire");
			if (attAutowire != null) {
				String auto = attAutowire.getValue();
				if ("byName".equals(auto)) {
					wire.append("\t\tautoWireByName(" + beanId + ");\n");
				}
				if ("byType".equals(auto)) {
					wire.append("\t\tautoWireByType(" + beanId + ");\n");
				}
			}
		

			if (!TransactionProxyFactoryBean.class.getName().equals(clz)) {
				fields.append(singleton + " " + clz + " " + beanId + ";\n");
			}
			
			Attribute attInit = bean.getAttribute("init-method");
			if (attInit != null) {
				wire.append("// XXX: " + beanId + "." + attInit.getValue() + "(?);\n");
			}
			Attribute attDestroy = bean.getAttribute("destroy-method");

			if (attDestroy != null) {
				wire.append("// XXX : call destroy method");
			}

			if (TransactionProxyFactoryBean.class.getName().equals(fullClassName)) {
				Attribute attAb = bean.getAttribute("abstract");
				if (attAb.getValue().equals("false")) {
					Element targetP = (Element) XPath.selectSingleNode(bean, "property[@name='target']");
					Element target = (Element) targetP.getChildren().get(0);
					if ("ref".equals(target.getName())) {
						Attribute attBean = target.getAttribute("bean");
						if (attBean != null) {
							String targetBeanName = attBean.getValue();
							if (topBeans.contains(targetBeanName)) {
								// change the class type to the target type
								Element te = (Element) XPath.selectSingleNode(jdomDocument, 
										"/beans/bean[@id=['" + targetBeanName + "']" );
								String className = te.getAttributeValue("class");
								fields.append("\tstatic " + 
										className.substring(className.lastIndexOf('.') + 1)
										+ " " + beanId + ";\n");
								replacers.put(targetBeanName, beanId);
							}
							else {
							}
						}
						else {
							String targetBeanName = target.getAttribute("local").getValue();
							if (topBeans.contains(targetBeanName)) {
								// change the class type to the target type
								String className = getBeanClass(jdomDocument, targetBeanName);
								fields.append("\tstatic " + 
										className.substring(className.lastIndexOf('.') + 1)
										+ " " + beanId + ";\n");
								
								replacers.put(targetBeanName, beanId);
							}
							else {
								throw new RuntimeException("local ref does not exis? " + targetBeanName);
							}
						}
						
					}
					else if("bean".equals(target.getName())) {
						String className = target.getAttributeValue("class");
						// create local bean
						wire.append("\t\t" + className + " var" + className.lastIndexOf('.' + 1));
						if (target.getChildren().size() > 0) {
							wire.append("// XXX: init the local bean!");
						}
					}
					wire.append("// XXX: enableTx(" + beanId + "....);\n");
				}
				// todo: deal with abstract
			}else {
				
				// dive into one level to the bean
				List beanKids = bean.getChildren();
				for (Iterator iterator = beanKids.iterator(); iterator.hasNext();) {
					Element p = (Element) iterator.next();
					if (p.getName().equals("constructor-arg")) {
						wire.append("// XXX: " + beanId + " = new " + clz + "(...);\n");
						continue;
					}
					
					// should be properties
					Attribute propAttr = p.getAttribute("name");
					String propName = propAttr.getValue();
					List pc = p.getChildren();
					Element propChild = (Element) pc.get(0);
					if (propChild.getName().equals("ref")) {
						Attribute attBean = propChild.getAttribute("bean");
						if (attBean != null) {
							String refbean = attBean.getValue();
							if (topBeans.contains(refbean)) {
								// get the refbena class
								String className = getBeanClass(jdomDocument, refbean);
								if (className.endsWith("FactoryBean") && 
										!className.equals(TransactionProxyFactoryBean.class.getName())) {
									wire.append("\t\t" + beanId + "." + getSetter(propName) + 
											"((" + guessType(refbean) + ")" + refbean + ");\n");
									// may need to use the pname to guess the type
								}
								else {
									wire.append("\t\t" + beanId + "." + getSetter(propName) + 
											"(" + refbean + ");\n");
								}
							}
							else {
								// this will display a casting error anyway
								wire.append("\t\t" + beanId + "." + getSetter(propName) + 
										"((" + guessType(refbean) + ")refBean(\"" + refbean + "\"));\n");
							}
						}
						else {
							Attribute attLocal = propChild.getAttribute("local");
							String local = attLocal.getValue();
							wire.append("\t\t" + beanId + "." + getSetter(propName) + "(" + local + ");\n");
						}
					}
					else if (propChild.getName().equals("bean")) {
						// local bean
						String localBeanClass = propChild.getAttributeValue("class");
						if (propChild.getChildren().size() == 0) {
							wire.append("\t\t" + beanId + "." + getSetter(propName) + 
									"(new " + localBeanClass + "());\n");
						}
						else {
							// need to create a local var first and call the setter
							// TODO: add auto logic here
							wire.append("\t\t" + localBeanClass + " " +
									propName + " = new " + localBeanClass + "();\n");
							addXXX(wire, "add init code here...");
							wire.append("\t\t" + beanId + "." + getSetter(propName) + 
									"("+ propName + ");\n");
						}
					}
					else if(propChild.getName().equals("value")) {
						String v = propChild.getText();
						wire.append("\t\t" + beanId + "." + getSetter(propName) + 
								"("+ v + ");\n");
					}
					else if(propChild.getName().equals("props")) {
						imports.append("import java.util.Properties;\n");
						String propsName = "prop_" + propName;
						wire.append("\t\tProperties " + propsName + " = new Properties(); {\n");
						List props = propChild.getChildren();
						for (Iterator iter1 = props.iterator(); iter1.hasNext();) {
							Element pe = (Element) iter1.next();
							String key = pe.getAttributeValue("key");
							String value = pe.getText();
							wire.append("\t\t\t" + propsName + ".put(\"" + key + "\", \"" + value + "\");\n");
						}
						wire.append("\t\t}\n");
						wire.append("\t\t" + beanId + "." + getSetter(propName) + "("+ propsName + ");\n");
						
					}
					else {
						addXXX(wire, "set the proper " + beanId + "." + propName + " with a " + propChild.getName());
					}
				}
			}
		}
		
		String fieldsString = fields.toString();
		String wireString = wire.toString();
		for (Iterator iter = replacers.keySet().iterator(); iter.hasNext();) {
			String tobeReplaced = (String) iter.next();
			String replacer = (String) replacers.get(tobeReplaced);
			fieldsString = fieldsString.replaceAll(tobeReplaced, replacer);
			wireString = wireString.replaceAll(tobeReplaced, replacer);
		}
		// remove the dups in fields
		String[] flds = fieldsString.split("\n");
		List fl = new ArrayList();
		for (int i = 0; i < flds.length; i++) {
			String f = flds[i].trim();
			if (!fl.contains(f)) {
				fl.add(f);
			}
		}
		fields = new StringBuffer();
		for (Iterator iter = fl.iterator(); iter.hasNext();) {
			String f = (String) iter.next();
			if (f.charAt(0) != '\t') {
				fields.append('\t').append(f).append('\n');
			}
			else {
				fields.append(f).append('\n');
			}
		}
		return new String[] {imports.toString(), fields.toString(), wireString};
	}

	public static String xml2JacnClass(File f) throws JDOMException, IOException  {
		String[] ss = ConvertUtil.xml2Jacn(f);
		StringBuffer sb = new StringBuffer();
		
		sb.append("import java.io.IOException;");
		sb.append('\n').append(ss[0]);
		sb.append('\n').append("public class JacnContext extends AbstractJacnConfigFile {");
		sb.append('\n').append(ss[1]);
		sb.append('\n').append("	public void wire() throws Exception {");
		
		sb.append('\n').append(ss[2]);
		sb.append('\n').append("	}");
		sb.append('\n').append("	public static void main(String[] args) throws IOException {\r\n" + 
				"		String x = bran.spring.jacn.ConvertUtil.jacn2Xml(JacnContext.class);\r\n" + 
				"		System.out.println(x);\r\n" + 
				"	}\r\n");
		sb.append('\n').append("}");
		return sb.toString();
	}
	private static void addXXX(StringBuffer wire, String s) {
		wire.append("// XXX: " + s + "\n");
	}

	private static String getBeanClass(org.jdom.Document jdomDocument, String targetBeanName) throws JDOMException {
		Element te = (Element) XPath.selectSingleNode(jdomDocument, 
				"/beans/bean[@id='" + targetBeanName + "']" );
		String className = te.getAttributeValue("class");
		return className;
	}

	private static String guessType(String refbean) {
		char fc = refbean.charAt(0);
		if (Character.isLowerCase(fc)) {
			fc = Character.toUpperCase(fc);
			return fc + refbean.substring(1);
		}
		else
			return refbean;
	}

	private static String getSetter(String pname) {
		char firstChar = pname.charAt(0);
		firstChar = Character.toUpperCase(firstChar);
		return "set" + firstChar + pname.substring(1);
	}

	static public void merge2Left(Element left, Element right) {
		List attrs = right.getAttributes();
		for (Iterator iter = attrs.iterator(); iter.hasNext();) {
			Attribute attr = (Attribute) iter.next();
			// not changing the id
			if (!attr.getName().equals("id")) {
				left.removeAttribute(attr.getName());
				left.setAttribute((Attribute) attr.clone());//getAttributes().add(attr.clone());
			}
		}
		
		List kids = right.getChildren();
		for (Iterator iter = kids.iterator(); iter.hasNext();) {
			Element e = (Element) iter.next();
			left.addContent((Element) e.clone());
		}
	}
}
