/*
 * $Log$
 * Revision 1.1  2006/07/02 04:54:40  branaway
 * factored out the inner classes from the JcnMethodVisitor
 *
 * Revision 1.32  2005/10/21 03:03:42  branaway
 * minor enhancement of SpringFaces
 *
 * Revision 1.31  2005/09/28 10:05:15  branaway
 * added support for generic AOP
 *
 * Revision 1.30  2005/09/27 14:58:55  branaway
 * added support for proxying prototype beans
 *
 * Revision 1.29  2005/09/26 16:02:10  branaway
 * added interface for better injection of implicit objects in the faces aware objects. added caching in the variable resolver. moving the implicit injection from jacn app contextto variable resolver.
 *
 * Revision 1.28  2005/09/24 17:58:15  branaway
 * added special method called bindValue for handlinggeneric #{el} expression
 *
 * Revision 1.27  2005/09/21 03:48:59  branaway
 * now the scoping of Jacn beans really work with the introduction o SpringFacesBeanFactory. SetJSf value binding on  bean properties also works.
 *
 * Revision 1.26  2005/09/20 14:56:43  branaway
 * added setWebScopeSession and setWebScopeRequest to the config file, so later on we can use that informaion in jacn unified JSF and Spring
 *
 * Revision 1.25  2005/09/08 14:53:37  branaway
 * fixed issues with .class in jdk5
 *
 * Revision 1.24  2005/09/06 15:03:30  branaway
 * force cglib for enableTx if no interface is specified for the target object
 *
 * Revision 1.23  2005/09/04 11:51:08  branaway
 * fixed the operand to use for external field reference
 *
 * Revision 1.22  2005/09/04 10:23:46  branaway
 * support referencing factorybean instance itself
 *
 * Revision 1.21  2005/08/25 22:49:33  branaway
 * added supported for ctor DI and setter DI mixed in any order
 *
 * Revision 1.20  2005/08/25 09:41:51  branaway
 * support antowire on transactional object
 *
 * Revision 1.19  2005/08/25 07:43:28  branaway
 * better support for ArrayList as regular bean
 *
 * Revision 1.18  2005/08/23 19:29:39  branaway
 * various fixes
 *
 * Revision 1.17  2005/08/23 05:08:02  branaway
 * added loadPlaceholderProperties() API to eliminate the use of PropertyPlaceholderFactoryBean
 *
 * Revision 1.16  2005/08/22 14:44:33  branaway
 * now local vars in the wire() method of jacnfig is always treated as inline beans (common collections exluded).
 *
 * Revision 1.15  2005/08/20 15:01:14  branaway
 * made the testEnableTx a real test against the DOM structure.
 * removed jDom from the jacn jar.
 *
 * Revision 1.14  2005/08/20 09:05:24  branaway
 * fixed issues when debug is off
 *
 * Revision 1.13  2005/08/19 19:21:31  branaway
 * added layInit. Added build.xml
 *
 * Revision 1.12  2005/08/10 12:09:20  branaway
 * deprecated invoke interface in the config interface. now arbitrary method invoking on beans are supported in the wire method
 *
 * Revision 1.11  2005/08/10 06:56:37  branaway
 * added support for implicitly using TransactionProxyFactoryBean
 *
 * Revision 1.10  2005/08/08 17:09:07  branaway
 * added support for adding post processing functional calls.
 * removed web dir since the web is now in another module called htdocs
 *
 * Revision 1.9  2005/08/04 17:58:03  branaway
 * works with AppFuse now
 *
 * Revision 1.8  2005/08/04 15:53:47  branaway
 * clean the comment
 *
 */
package bran.spring.jacn;

import java.util.ArrayList;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Label;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;
import org.objectweb.asm.util.ASMifierAbstractVisitor;

/**
 * A {@link MethodVisitor} that, together with {@link JacnClassVisitor}, constructs the 
 * dependency relationship of Jacn beans and presents the result in DOM document. 
 * 
 * this one is only intended for processing the destroy() method.
 * 
 * @author bran
 * @version 0.5
 */

public class DestroyMethodVisitor extends ASMifierAbstractVisitor implements MethodVisitor, Opcodes {
	String methodName;
	int access;

	int lastOpcode = -1;

	private JacnClassVisitor springConfig;

	private FieldElementObj operand; 

	public DestroyMethodVisitor(JacnClassVisitor springConfig, String name, int access, String desc) {
		super("destroy method visitor");
		this.springConfig = springConfig;
		methodName = name;
		this.access = access;
	}

	public AnnotationVisitor visitAnnotationDefault() {
		notValid("visitAnnotationDefault");
		return null;
	}

	public AnnotationVisitor visitParameterAnnotation(final int parameter,
			final String desc, final boolean visible) {
		notValid("visitParameterAnnotation");
		return null;
	}

	public void visitCode() {
	}

	public void visitVarInsn(final int opcode, int varIndex) {
	}
	
	public void visitTypeInsn(final int opcode, final String desc) {
	}

	public void visitFieldInsn(final int opcode, final String owner,
			final String name, final String desc) {
			switch (opcode) {
			case PUTFIELD:
				break;
			case GETFIELD:
			case GETSTATIC:
					if (owner.equals(this.springConfig.className)){
						this.operand = getFieldElement(name);
					}
					else {
					}
				break;
			case PUTSTATIC: 
				break;
			default:
				throw new RuntimeException("unexpected case");
			}
		lastOpcode = opcode;
	}

	/**
	 * @param name
	 * @return
	 */
	private FieldElementObj getFieldElement(String name) {
		return (FieldElementObj) springConfig.fldElems.get(name);
	}


	/**
	 * method calls
	 * 
	 * the only allowed statement in destroy() is simple function calls on fields
	 * 
	 * e.g.:  foo.close(); bar.dispose();
	 * 
	 */
	public void visitMethodInsn(final int opcode, final String owner,
			final String methodName, final String desc) {
		
		
		Type[] args = Type.getArgumentTypes(desc);
		if (args.length > 0) {
			throw new RuntimeException("destroy methods must not take arguments");
		}
		
		Type returnType =  Type.getReturnType(desc);
		if (returnType != Type.VOID_TYPE) {
			throw new RuntimeException("destroy methods must return void");
		}
		
		switch (opcode) {
		case INVOKEINTERFACE:
		case INVOKESTATIC:
		case INVOKEVIRTUAL:
		{
			this.operand.setDestroyMethod(methodName);
			break;
		}
		case INVOKESPECIAL:  // <init> call
		{
			break;
		}
		default:
			throw new RuntimeException("unexpected case");
		}
		lastOpcode = opcode;
	}


	public void visitIincInsn(final int var, final int increment) {
		notValid("visitIincInsn");
	}

	public void visitTableSwitchInsn(final int min, final int max,
			final Label dflt, final Label labels[]) {
		notValid("visitTableSwitchInsn");
	}

	public void visitLookupSwitchInsn(final Label dflt, final int keys[],
			final Label labels[]) {
		notValid("visitLookupSwitchInsn");
	}


	public void visitMultiANewArrayInsn(final String desc, final int dims) {
		notValid("visitMultiANewArrayInsn");
	}

	public void visitTryCatchBlock(final Label start, final Label end,
			final Label handler, final String type){
		//notValid("visitTryCatchBlock");
		// ignore
	}

	public void visitLocalVariable(final String name, final String desc,
			final String signature, final Label start, final Label end,
			final int index) {
	}

	public void visitLineNumber(final int line, final Label start) {
		lastOpcode = -1;
	}

	public void visitMaxs(final int maxStack, final int maxLocals) {
		lastOpcode = -1;
	}

	public void visitEnd() {
		// does nothing
	}

	public String getMethodName() {
		return methodName;
	}

	public void setMethodName(String methodName) {
		this.methodName = methodName;
	}
	private void notValid(String msg) {
		throw new RuntimeException("instruction not supported: " + msg);
	}
	
	ArrayList opStack = new ArrayList();

	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitInsn(int)
	 */
	public void visitInsn(int arg0) {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitIntInsn(int, int)
	 */
	public void visitIntInsn(int arg0, int arg1) {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitJumpInsn(int, org.objectweb.asm.Label)
	 */
	public void visitJumpInsn(int arg0, Label arg1) {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitLabel(org.objectweb.asm.Label)
	 */
	public void visitLabel(Label arg0) {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see org.objectweb.asm.MethodVisitor#visitLdcInsn(java.lang.Object)
	 */
	public void visitLdcInsn(Object arg0) {
		// TODO Auto-generated method stub
		
	}
	

}
