/**
 * @author bran
 */
package bran.spring.jacn;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Map;

import org.jdom.JDOMException;
import org.objectweb.asm.ClassReader;
import org.objectweb.asm.tree.ClassNode;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanDefinitionStoreException;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.xml.XmlBeanDefinitionParser;
import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;
import org.springframework.core.io.Resource;
import org.w3c.dom.Document;

/**
 * Bean definition reader for JAC bean definitions. 
 *
 * <p>This class loads a DOM document from class file and applies 
 * the bean definition parser to it.
 * 
 * @see XmlBeanDefinitionParser
 */
public class JacnBeanDefinitionReader extends XmlBeanDefinitionReader {

	Document doc;
	private String xmlText;

	PropertyChangeListener pcl;
	
	public PropertyChangeListener getPcl() {
		return pcl;
	}

	public void setPcl(PropertyChangeListener pcl) {
		this.pcl = pcl;
	}

	public JacnBeanDefinitionReader(BeanDefinitionRegistry arg0) {
		super(arg0);
	}

	public int loadBeanDefinitions(Resource resource) throws BeansException {
		if (resource == null) {
			throw new BeanDefinitionStoreException("resource cannot be null");
		}

		InputStream is = null;
		try {
			is = resource.getInputStream();
		    doc = getDom(is);
			return registerBeanDefinitions(doc, resource);
		}
		catch (IOException ex) {
			throw new BeanDefinitionStoreException(
					"IOException parsing XML document from " + resource, ex);
		} catch (JDOMException e) {
			throw new BeanDefinitionStoreException(
					"JDOMException building XML document from " + resource, e);
		}
		
		finally {
			if (is != null) {
				try {
					is.close();
				}
				catch (IOException ex) {
				}
			}
		}
	}


	private Document getDom(InputStream is) throws IOException, JDOMException {
		ClassReader cr;

		/*
		 * // let's copy the inputstream. We'll be using it twice ArrayList al =
		 * new ArrayList(10000);
		 * 
		 * int b = is.read(); while (b != -1) { al.add(new Byte((byte) b)); b =
		 * is.read(); } byte[] ba = new byte[al.size()]; for (int i = 0; i <
		 * ba.length; i++) { ba[i] = ((Byte)al.get(i)).byteValue(); }
		 * 
		 * ByteArrayInputStream bais = new ByteArrayInputStream(ba); is.close();
		 * 
		 * cr = new ClassReader(bais);
		 * 
		 * ClassNode cn = new ClassNode(); cr.accept(cn, false);
		 * 
		 * bais.reset();
		 */
		try {
			cr = new ClassReader(is);
			JacnClassVisitor visitor = new JacnClassVisitor();
			cr.accept(visitor, false);
			this.xmlText = visitor.getXml();

			if (pcl != null) {
				pcl.propertyChange(new PropertyChangeEvent(this, "xmlTest",
						null, xmlText));
			}
			
			this.beanScopeMap = visitor.getBeanScopeMap();
			
			return visitor.getDOM();
		} finally {
			is.close();
		}
	}


	public Document getDoc() {
		return doc;
	}


//	public void setDoc(Document doc) {
//		this.doc = doc;
//	}
//

	public String getXmlText() {
		return xmlText;
	}

	public void setXmlText(String xmlText) {
		this.xmlText = xmlText;
	}

	Map beanScopeMap;

	public Map getBeanScopeMap() {
		return beanScopeMap;
	}
	
}
