/**
 * $Log$
 * Revision 1.17  2006/07/02 04:46:54  branaway
 * support destroy and init mechanism: using different method visitors for different wire method
 *
 * Revision 1.16  2005/11/06 13:59:31  branaway
 * added better support for concurrent flows with transaction enabled backing beans
 *
 * Revision 1.15  2005/09/27 14:58:55  branaway
 * added support for proxying prototype beans
 *
 * Revision 1.14  2005/09/26 16:02:10  branaway
 * added interface for better injection of implicit objects in the faces aware objects. added caching in the variable resolver. moving the implicit injection from jacn app contextto variable resolver.
 *
 * Revision 1.13  2005/09/20 14:56:43  branaway
 * added setWebScopeSession and setWebScopeRequest to the config file, so later on we can use that informaion in jacn unified JSF and Spring
 *
 * Revision 1.12  2005/08/25 07:43:28  branaway
 * better support for ArrayList as regular bean
 *
 * Revision 1.11  2005/08/22 14:44:33  branaway
 * now local vars in the wire() method of jacnfig is always treated as inline beans (common collections exluded).
 *
 * Revision 1.10  2005/08/20 09:05:24  branaway
 * fixed issues when debug is off
 *
 * Revision 1.9  2005/08/10 06:56:37  branaway
 * added support for implicitly using TransactionProxyFactoryBean
 *
 * Revision 1.8  2005/08/08 17:09:07  branaway
 * added support for adding post processing functional calls.
 * removed web dir since the web is now in another module called htdocs
 *
 * Revision 1.7  2005/08/04 17:58:03  branaway
 * works with AppFuse now
 *
 * Revision 1.6  2005/08/02 22:04:47  branaway
 * API change: singletons requires using static field in the config file.
 *
 * added more test cases.
 *
 * Revision 1.5  2005/08/02 17:23:11  branaway
 * added <Class>.class.getName() support. Changed the config file interface name.
 *
 * Revision 1.4  2005/07/25 11:57:34  branaway
 * added system id to doctype
 *
 * @author bran
 */
package bran.spring.jacn;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.output.DOMOutputter;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;
import org.objectweb.asm.util.AbstractVisitor;

/**
 * A class visitor that iterates thru class files structure.  Most of the hardwork is done in
 * the {@link JacnMethodVisitor}. 
 * @author bran
 *
 */
public class JacnClassVisitor extends AbstractVisitor implements ClassVisitor, FieldVisitor,
		AnnotationVisitor, Opcodes{
	String className; // such as "com/mycompany.Myclass"
	
	String[] interfaces;

	Document xmlDoc; // = new Document(new Element("beans"));

	JacnMethodVisitor mv;
	
	/**
	 * keep track of all the flds and operations on them
	 * the key is the field name, the value is {@link FieldElement}
	 */
	HashMap fldElems = new HashMap();

	/**
	 * keep track of all the synthesized fields
	 * the key is the field name, the value is {@link FieldElement}
	 */
	HashMap synthesizedfldElems = new HashMap();

	/**
	 * keep track of local vars in all methods
	 * The key is the method name, the scope where the local defined. 
	 * the value is an List of VarDoc 
	 */
	HashMap methodLocalVars = new HashMap();

//	private String currentMethod;
//
//	private Element tmpElement;

//	ClassNode classNode;


	private boolean builtAlready = false;

	/*	public JacnClassVisitor(ClassNode cn) {
		this.classNode = cn;
		xmlDoc = new Document(new Element("beans"));
		xmlDoc.setDocType(
				new DocType(
				"beans", "-//SPRING//DTD BEAN//EN",
				"http://www.springframework.org/dtd/spring-beans.dtd"));
	}
*/
	public void visit(int version, int access, String name, String signature,
			String superName, String[] interfaces) {
		className = name;
		this.interfaces = interfaces;
	}

	public void visitSource(String source, String debug) {
	}

	public void visitOuterClass(String owner, String name, String desc) {
	}

	public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
		return this;
	}

	public void visitAttribute(Attribute attr) {
	}

	public void visitInnerClass(String name, String outerName,
		String innerName, int access) {
	}

	/**
	 * store synthesized fields for the .class syntax
	 * [fieldname, Class object]
	 */
//	HashMap dotClassNames = new HashMap();
	
	
	public FieldVisitor visitField(int access, String name, String desc,
			String signature, Object value) {
		String type = ConvertUtil.getType(desc);

		Class c;
		try {
			c = Class.forName(type, false, this.getClass().getClassLoader());
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e.getMessage());
		}
		
		// Oh well, the following snippet is flawed. When the jacn class is a static inner class
		// the .class syntax will result in synth fields defined in th outer class
		// th JacnMethodVisitor should do the checking at the field acess.
		if ((access & Opcodes.ACC_SYNTHETIC) != 0 && (access & Opcodes.ACC_STATIC) != 0) {
			if (type.equals("java.lang.Class")) {
				return null;
//				// synthed field for .class syntax
//				dotClassNames.put(name, type);
//				if (Character.isDigit(name.charAt(name.length() - 1))) {
//					if (compilerHint != -1 && compilerHint != ECLIPSEJDT) {
//						throw new RuntimeException("The Jacn is confused by the compiler type: " +
//								"the naming convention of the synth class field is inconsistant");
//					}
//					else {
//						compilerHint = ECLIPSEJDT;
//					}
//				}
//				else {
//					if (compilerHint != -1 && compilerHint != SUNJDK) {
//						throw new RuntimeException("The Jacn is confused by the compiler type: " +
//								"the naming convention of the synth class field is inconsistant");
//					}
//					else {
//						compilerHint = SUNJDK;
//					}
//				}
//				// TODO: other compilers: jrocket, jike, etc...
			}
		}
		else if(IJacnConfigFile.class.isAssignableFrom(c)){
			// ignore
		}
		else if (!name.startsWith("_")) {
			FieldElementObj fe = new FieldElementObj(desc, name);
			if ((access & Opcodes.ACC_STATIC) == 0) {
				fe.setSingleton(false);
				getBeanScopeMap().put(name, WebScope.NONE_SCOPE);
			}
			else {
				fe.setSingleton(true);
				getBeanScopeMap().put(name, WebScope.APPLICATION_SCOPE);
			}
			//xmlDoc.getRootElement().addContent(fe.e);
			fldElems.put(name, fe); // also keep it in the hashmap for the field
		}
		return this;
	}

	/**
	 * todo handle arrays
	 * @param slashName
	 * @return
	 */
	public static String dot(String slashName) {
		if (slashName.startsWith("L")) { //"L....;"
			slashName = slashName.substring(1, slashName.length() - 1);  //
		}
		return slashName.replace('/', '.');
	}

	public MethodVisitor visitMethod(int access, String methName, String desc,
			String signature, String[] exceptions) {
//		currentMethod = methName;
		if (methName.equals("init")) {
			// TODO: add signature checking
			return new InitMethodVisitor(this, methName, access, desc);
		}
		else if (methName.equals("destroy")) {
			// TODO: add signature checking
			return new DestroyMethodVisitor(this, methName, access, desc);
		}
		else if (methName.equals("wire") ) {
			// TODO: add signature checking
			return new JacnMethodVisitor(this, methName, access, desc);
		}

		else if (methName.equals("<init>")) {
			return null;
		}
		else if((access & Opcodes.ACC_SYNTHETIC) != 0 && (access & Opcodes.ACC_STATIC) != 0) {
			return null;
		}
		else {
			//throw new RuntimeException("method not supported: " + methName);
			// ignored
			return null;
		}
	}

	public void visitEnd() {
	}


	public void visit(String name, Object value) {
	}

	public void visitEnum(String name, String desc, String value) {
	}

	public AnnotationVisitor visitAnnotation(String name, String desc) {
		return this;
	}

	public AnnotationVisitor visitArray(String name) {
		return this;
	}

	public static String toXml(Document doc) {
		XMLOutputter xo = new XMLOutputter();
		//xo.
		Format f = Format.getPrettyFormat();
		xo.setFormat(f);
//		StringWriter sw = new StringWriter();
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		try {
			xo.output(doc, bos);
			//return new String(bos.toByteArray(), "ISO-8859-1");
			return new String(bos.toByteArray(), "UTF-8");
		} catch (IOException e) {
			e.printStackTrace();
			return "";
		}
	}

	void buildDoc() {
		if (!this.builtAlready) {
			xmlDoc = new Document(new Element("beans"));
			xmlDoc.setDocType(new DocType("beans", "-//SPRING//DTD BEAN//EN",
					"http://www.springframework.org/dtd/spring-beans.dtd"));
			Collection values = this.fldElems.values();
			for (Iterator iter = values.iterator(); iter.hasNext();) {
				FieldElementObj f = (FieldElementObj) iter.next();
				if (!f.name.startsWith("class$")) {
					Element fe = (Element) f.e.detach();
					if (fe.getName().equals("list")) {
						fe.setName("bean");
						fe.setAttribute("class", f.getClassName());
					}

					xmlDoc.getRootElement().addContent(fe);
					
					synchScope(f);
				}
			}
			values = this.synthesizedfldElems.values();
			for (Iterator iter = values.iterator(); iter.hasNext();) {
				FieldElementObj f = (FieldElementObj) iter.next();
				if (!f.name.startsWith("class$")) {
					Element fe = (Element) f.e.detach();
					if (fe.getName().equals("list")) {
						fe.setName("bean");
						fe.setAttribute("class", f.getClassName());
					}
					xmlDoc.getRootElement().addContent(fe);
					
					synchScope(f);
				}
			}
			
			postProcess();
		}
		this.builtAlready = true;
		
	}

	/**
	 * make sure the renamed target class retains the original scope attribute
	 * @param f
	 */
	public void synchScope(FieldElementObj f) {
		String bname = f.getName();
		if (bname.endsWith(PROXY_TARGET_APPENDIX)) {
			// synchronize the web scope with the proxy
			String proxyName = bname.substring(0, bname.length() - PROXY_TARGET_APPENDIX.length());
			this.beanScopeMap.put(bname, beanScopeMap.get(proxyName));
		}
	}

	private void postProcess() {
		// remove excessive id attribute from second level
		List es = xmlDoc.getRootElement().getChildren();
		for (Iterator iter = es.iterator(); iter.hasNext();) {
			Element e = (Element) iter.next();
			List kids = e.getChildren();
			for (Iterator iterator = kids.iterator(); iterator.hasNext();) {
				Element k = (Element) iterator.next();
				removeIdRecursive(k);
			}
		}
	}
	
	private void removeIdRecursive(Element e) {
		//if (!e.getName().equals("bean"))
			e.removeAttribute("id");
		List es = e.getChildren();
		for (Iterator iter = es.iterator(); iter.hasNext();) {
			Element ee = (Element) iter.next();
			removeIdRecursive(ee);
		}
	}

	public org.w3c.dom.Document getDOM() throws JDOMException {
		buildDoc();
		DOMOutputter dommer = new DOMOutputter();
		return dommer.output(xmlDoc);
	}

	public String getXml() {
		buildDoc();
		return toXml(xmlDoc);
	}

	public Document getXmlDoc() {
		buildDoc();
		return xmlDoc;
	}

	public Map getBeanScopeMap() {
		return beanScopeMap;
	}

	Map beanScopeMap = new HashMap();

	static public String PROXY_TARGET_APPENDIX  = "__target";;
}
