package bran.spring.jacn;

import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.DOMBuilder;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.beans.factory.xml.ResourceEntityResolver;
import org.springframework.beans.factory.xml.XmlBeanDefinitionReader;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.web.context.support.XmlWebApplicationContext;

import bran.spring.jacn.faces.FacesContextAware;
import bran.spring.jacn.faces.HttpSessionAware;
import bran.spring.jacn.faces.RequestParametersAware;
import bran.spring.jacn.faces.RequestScopeAware;
import bran.spring.jacn.faces.SessionScopeAware;

public class JacnWebApplicationContext extends XmlWebApplicationContext {
	private static final Log log = LogFactory.getLog(JacnWebApplicationContext.class);

	private static final String CLASS_EXT = ".class";

	private JacnBeanDefinitionReader jacnReader;

	PropertyChangeListener pcl;

	// contains <beanName, scope>
	HashMap beanScopeMap = new HashMap();
	// convenient store of bean name and class information
	HashMap beanNameClassMap = new HashMap();

	// contains the xml Documents (jdom) for all the jacn configs
	List beanXmlDocs = new ArrayList();

	public PropertyChangeListener getPcl() {
		return pcl;
	}

	public void setPropertyChangeListener(PropertyChangeListener listener) {
		this.pcl = listener;
	}

	protected void loadBeanDefinitions(DefaultListableBeanFactory beanFactory)
			throws IOException, BeansException {
		jacnReader = new JacnBeanDefinitionReader(beanFactory);
		XmlBeanDefinitionReader xmlReader = new XmlBeanDefinitionReader(
				beanFactory);
		xmlReader.setEntityResolver(new ResourceEntityResolver(this));

		if (this.pcl != null) {
			jacnReader.setPcl(getPcl());
		}
		jacnReader.setResourceLoader(this);
		String[] configLocations = getConfigLocations();

		if (configLocations != null) {
			for (int i = 0; i < configLocations.length; i++) {
				String loc = configLocations[i];
				Resource[] resources = getResources(loc);
				if (loc.endsWith(CLASS_EXT)) {
					jacnReader.loadBeanDefinitions(resources);
					Map m = jacnReader.getBeanScopeMap();
					beanScopeMap.putAll(m);
					DOMBuilder builder = new DOMBuilder();
					org.jdom.Document jdomDoc = builder.build(jacnReader
							.getDoc());
					beanXmlDocs.add(jdomDoc);
				} else if (loc.endsWith("xml")) {
					xmlReader.loadBeanDefinitions(resources);
				}
			}
		}
	}

	/**
	 * passing in  the parent is required to get the hierarchical context working
	 */
	protected final DefaultListableBeanFactory createBeanFactory() {
		return new SpringFacesBeanFactory(this, getInternalParentBeanFactory());
	}

	public void setConfigLocations(String[] configLocations) {
		// do some convenient transforming
		for (int i = 0; i < configLocations.length; i++) {
			String string = configLocations[i];
			if (string.endsWith(CLASS_EXT)) {
				// make sure the location form is: aaa/vvv/sss.class
				string = string.substring(0, string.length()
						- CLASS_EXT.length());
				string = string.replace('.', '/') + CLASS_EXT;
				configLocations[i] = string;
			} else if (string.endsWith("xml")) {
				// do nothing
			} else {
				if (string.startsWith(ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX)
						|| string.startsWith(ResourceLoader.CLASSPATH_URL_PREFIX)) {
					// let's assume its a jacn class
					string = string.replace('.', '/') + CLASS_EXT;
					configLocations[i] = string;
				} else {
					string = string.replace('.', '/') + CLASS_EXT;
					configLocations[i] = ResourceLoader.CLASSPATH_URL_PREFIX + string;
				}
			}
		}
		super.setConfigLocations(configLocations);
	}

	public String getScope(String beanName) {
		return (String) beanScopeMap.get(beanName);
	}

//	public Object getBean(String beanName) throws BeansException {
////		log.info(beanName);
//		
//		Object bean = super.getBean(beanName);
//
//		// processing generic EL expression value binding
////		if (WebScope.APPLICATION_SCOPE.equals(scope)) {
////		} else if (WebScope.SESSION_SCOPE.equals(scope)) {
////			Element bn = getBeanDefNode(beanName);
////		} else if (WebScope.REQUEST_SCOPE.equals(scope)) {
////		}
//		
//		return bean;
//	}
//
//	private Element getBeanDefNode(String beanName) {
//		for (Iterator iter = beanXmlDocs.iterator(); iter.hasNext();) {
//			Document doc = (Document) iter.next();
//			List beans = doc.getRootElement().getChildren();
//			for (Iterator iterator = beans.iterator(); iterator.hasNext();) {
//				Element b = (Element) iterator.next();
//				if (b.getAttributeValue("name").equals(beanName)) {
//					return b;
//				}
//			}
//		}
//		return null;
//
//	}

	// see if we use search for HSF value bindings directly
	// leave it with doTypeConversionIfNecessary. too much trouble..
	//
	// PropertyValue[] pvs =
	// beanDefinition.getPropertyValues().getPropertyValues();
	// for (int i = 0; i < pvs.length; i++) {
	// PropertyValue pvalue = pvs[i];
	// if (pvalue.getClass() == String.class) {
	// String v = ((String) pvalue.getValue()).trim();
	// if (v.startsWith("#{") && v.endsWith("}")) {
	// ApplicationFactory factory = (ApplicationFactory)
	// FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
	// if (factory != null) {
	// Application facesApplication = (factory).getApplication();
	// if (facesApplication != null) {
	// String jsfValue = (String)
	// facesApplication.createValueBinding(v).getValue(FacesContext.getCurrentInstance());
	// // set on the bean
	// // need to get the setter etc... too complicated
	// }
	// }
	// }
	//			
	// }
	// }

}
