package bran.spring.jacn;

import java.util.Iterator;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;
import org.springframework.aop.framework.ProxyFactoryBean;


public class ProxyFactoryField extends FieldElementObj {

	public static ProxyFactoryField proxy(FieldElementObj target) {
		return new ProxyFactoryField(target);
	}

	protected FieldElementObj target;

	public static final String INTER_PROP = "interceptorNames";

	public ProxyFactoryField(String classDesc, String id, boolean isSynthesized) {
		super(classDesc, id, isSynthesized);
	}

	private ProxyFactoryField(FieldElementObj target) {
		super(ProxyFactoryBean.class.getName(), target.name, false);

		Attribute idAttr = target.e.getAttribute("id");
		String oldId = idAttr.getValue();
		String newId = oldId + JacnClassVisitor.PROXY_TARGET_APPENDIX;
		idAttr.setValue(newId);
		target.name = newId;

		this.target = target;
		this.name = oldId;

		e = new Element("bean");
		e.getAttributes().add(new Attribute("id", name));
		e.getAttributes().add(
				new Attribute("class", ProxyFactoryBean.class.getName()));

		Element singletonEle = new Element("property");
		singletonEle.getAttributes().add(new Attribute("name", "singleton"));
		Element v = new Element("value");
		if (target.isSingleton()) {
			v.setText("true");
		} else {
			v.setText("false");
		}
		singletonEle.addContent(v);
		e.addContent(singletonEle);

		Element tarEle = new Element("property");
		tarEle.getAttributes().add(new Attribute("name", "targetName"));
		Element refe = new Element("value");
		refe.setText(target.name);
		tarEle.addContent(refe);
		e.addContent(tarEle);
	}

	public void addProxyInterfaces(ArrayObject interfaces) {
		if (interfaces != null && interfaces.getArrayLength() > 0) {
			Element te = new Element("property");
			te.getAttributes().add(new Attribute("name", "proxyInterfaces"));
			te.addContent(interfaces.e);
			e.addContent(te);
		} else {
			// force using cglib
			// ps. will break confluence bad code. doing nothing for now
			Element te = new Element("property");
			te.getAttributes().add(new Attribute("name", "proxyTargetClass"));
			te.addContent(ConvertUtil.newValueElement("true"));
			e.addContent(te);
		}
	}

	public void addInterceptor(FieldElementObj inter) {
		if (inter != null) {
			XPath xpath;
			try {
				xpath = XPath.newInstance("./property[@name='" + INTER_PROP
						+ "']");
				Element p = (Element) xpath.selectSingleNode(e);
				if (p == null) {
					p = new Element("property");
					p.getAttributes().add(new Attribute("name", INTER_PROP));
					Element le = new Element("list");
					p.addContent(le);
					e.addContent(p);
				}
				Element plist = (Element) XPath.newInstance("list")
						.selectSingleNode(p);
				Element pv = new Element("value");
				pv.setText(inter.name);
				plist.addContent(pv);

			} catch (JDOMException e1) {
				throw new RuntimeException("addInterceptor went wrong: " + e1);
			}
		}
	}

	public void addInterceptor(ArrayObject advices) {
		if (advices == null) {
			return;
		}
		List refs = advices.e.getChildren();
		// each of it must be <ref bean="..."/>
		for (Iterator iter = refs.iterator(); iter.hasNext();) {
			Element e = (Element) iter.next();
			if (e.getName().equals("ref")) {
				String adviceName = e.getAttributeValue("bean");
				FieldElementObj bogus = new FieldElementObj(String.class.getName(), adviceName);
				this.addInterceptor(bogus);
			}
			else {
				throw new RuntimeException("interceptors must be fields.");
			}
		}
	}

	public XmlElementObject getTarget() {
		return target;
	}

	public void setTarget(FieldElementObj target) {
		this.target = target;
	}

	public void autoWireByName() {
		target.e.getAttributes().add(new Attribute("autowire", "byName"));
	}

	public void autoWireByType() {
		target.e.getAttributes().add(new Attribute("autowire", "byType"));
	}

	public void lazyInit() {
		e.getAttributes().add(new Attribute("lazy-init", "true"));
		target.e.getAttributes().add(new Attribute("lazy-init", "true"));
	}

	/**
	 * an operand is assigned to the target
	 * 
	 * @param xo
	 */
	public void assignedValue(XmlElementObject xo) {
		// should merge, rather than replace
		ConvertUtil.merge2Left(target.e, xo.e);
		// target.e.setAttribute("class", getClassName());
	}
}
