package bran.spring.jacn.faces;

import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public abstract class AbstractFlowController implements FlowController, RequestParametersAware, FacesContextAware {
	private static final long serialVersionUID = 1978333182379921068L;
	private static final Log log = LogFactory.getLog(AbstractFlowController.class);

	transient protected Map params;
	protected FlowManager flowManager;
	protected String homeView = "";
	protected String currentView;
	transient protected FacesContext facesContext;

	public String end() {
		return this.flowManager.returnToParent(this);
	}

	public void setRequestParameters(Map requestParamsMap) {
		this.params = requestParamsMap;
	}

	public void setFlowManager(FlowManager flowManager) {
		this.flowManager = flowManager;
	}

	public String begin(Object arg) {
		return getHomeView();
	}

	public String resume(FlowController childFlow) {
		return getCurrentView();
	}

	private String getCurrentView() {
		return currentView == null? getHomeView() : currentView;
	}

	public abstract String getHomeView() ;

	public void setHomeView(String vid) {
		this.homeView = vid;
	}

	public void setFacesContext(FacesContext facesContext) {
		this.facesContext = facesContext;
	}
	
	/**
	 * record and return the view id, for better default behavior of resume()
	 * @param viewid
	 * @return
	 */
	protected String view(String viewid) {
		this.currentView = viewid;
		return viewid;
	}

	public Map getParams() {
		return params;
	}

	public void setParams(Map params) {
		this.params = params;
	}

	public FacesContext getFacesContext() {
		return facesContext;
	}

	public FlowManager getFlowManager() {
		return flowManager;
	}

	public void setCurrentView(String currentView) {
		this.currentView = currentView;
	}
	
	public void addMessage(String s) {
		facesContext.addMessage(null, new FacesMessage(s));		
	}

	public String getRequestParam(String string) {
		return (String) getParams().get(string);
	}
	
	/**
	 * push the bean to the flow scope store, effectively making the bean visible 
	 * to views in current flow and sub flows. A.K.A. outjection in Seam terms.
	 * @param name
	 * @param bean
	 */
	public void makeVisible(String name, Object bean) {
		flowManager.setBean(name, bean);
	}

	protected String redirect(String vid) {
		return FlowManager.REDIRECT + vid;
	}
}
