package bran.spring.jacn.faces;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.jsf.FacesContextUtils;

import bran.spring.jacn.JacnClassVisitor;
import bran.spring.jacn.JacnWebApplicationContext;

public class FlowManager implements Serializable {
	public static final String REDIRECT = "redirect:";

	public static final int REQUEST_TTL = 10000;

	private static final Log log = LogFactory.getLog(FlowManager.class);

	private static final long serialVersionUID = 1L;

	// Map store = new HashMap(); // [id, flow]

	Map flowInstanceNumMap = new HashMap(); // [localflowid, Counter]

	// private Map resumePoints = new HashMap();

	// 
	FlowTree flowStack = new FlowTree();

	// FlowStack flowStack = new FlowStack(); // [Flow]

	private String homeViewId;

	private String viewHint;

	private List redirectList = Collections.synchronizedList(new ArrayList());

	private String startingFlowName = "";

	public static final String FLOW_MANAGER = "_flow_manager_";

	public static final String FLOW_ID = "_flowid_";

	public static FlowManager getCurrentInstance(FacesContext context) {
		Map sessionMap = context.getExternalContext().getSessionMap();
		JacnWebApplicationContext jacnCtx = (JacnWebApplicationContext) FacesContextUtils
		.getRequiredWebApplicationContext(context);
		
		FlowManager fm = (FlowManager) sessionMap.get(FLOW_MANAGER);

		if (fm == null) {
			log.debug("FlowManager not in session map. get a new copy");
			Map fms = jacnCtx.getBeansOfType(FlowManager.class);
			if (fms.size() == 0) {
				fm = new FlowManager();
			} else if (fms.size() > 1) {
				log.warn("found multiple deployments of FlowManager. Use the first one.");
				fm = (FlowManager) fms.get(fms.keySet().iterator().next());
			} else {
				fm = (FlowManager) fms.get(fms.keySet().iterator().next());
			}

			sessionMap.put(FLOW_MANAGER, fm);
		}
		// fm.setWctx(jacnCtx);
		return fm;
	}

	public static FlowManager getCurrentInstanceWithoutCreation(FacesContext context) {
		Map sessionMap = context.getExternalContext().getSessionMap();
		JacnWebApplicationContext jacnCtx = (JacnWebApplicationContext) FacesContextUtils
				.getRequiredWebApplicationContext(context);

		FlowManager fm = (FlowManager) sessionMap.get(FLOW_MANAGER);
		return fm;
	}
	
	public void setContextFlowId(String fid) {
		Map requestMap = FacesContext.getCurrentInstance().getExternalContext()
				.getRequestMap();
		if (fid == null || fid.equals("")) {
			requestMap.remove(FLOW_ID);
		} else {
			requestMap.put(FLOW_ID, fid);
		}
	}

	public String getContextFlowPath() {
		return (String) FacesContext.getCurrentInstance().getExternalContext()
				.getRequestMap().get(FLOW_ID);
	}

	/**
	 * pass in the flow controller's name to generated a flow id
	 * 
	 * @param flowName
	 * @return a local flow id. it's part of the full flow id path.
	 */
	public synchronized String getNewFlowId(String flowName) {
		Long c = (Long) this.flowInstanceNumMap.get(flowName);
		if (c == null) {
			c = new Long(0);
			flowInstanceNumMap.put(flowName, c);
		}
		Long next = new Long(c.longValue() + 1);
		flowInstanceNumMap.put(flowName, next);
		return flowName + "-" + c;
	}

	// private void storeResumePoint(FlowController subFlow, String resumePoint)
	// {
	// synchronized (resumePoints) {
	// ((Map) resumePoints).put(subFlow, resumePoint);
	// }
	// }

	/**
	 * return to the parent flow of the current flow on the stack 
	 * 
	 * @return the resume point (a view id) as determined by the parent flow controller
	 */
	public String returnToParent() {
		Flow currentFlow = flowStack.popFlow();
		FlowController cfc = currentFlow.getFlowController();
		return resumeCurrent(cfc);
	}

	/**
	 * resume to the parent flow of the specified flow controller
	 * it recursively matches the flow controller on the current flow stack and
	 * resume to the parent after discarding all the offsprings of the parent.
	 * 
	 * This is to allow the flow logic to drop to a specific flow stack frame. 
	 *   
	 * @return the resume point (a view id) as determined by the parent flow controller
	 */
	public String returnToParent(FlowController flowCon) {
		// 
		Flow currentFlow = flowStack.popFlowUpto(flowCon);
		if (currentFlow == null ) {
			// could not matach a flow
			return redirectView(getHomeViewId()); // shall this be redirect?
		}
		else {
			return resumeCurrent(flowCon);
		}
	}
	
	/**
	 * resume the flow on the top of the stack
	 * @param kidFlow
	 * @return
	 */
	private String resumeCurrent(FlowController kidFlow) {
		Flow parentFlow = flowStack.peekFlow();
		if (parentFlow != null) {
			FlowController parentFlowController = parentFlow.getFlowController();
			// use redirect for returning from child flow
			return redirectView(parentFlowController.resume(kidFlow));
		}
		else {
			return redirectView(getHomeViewId()); // shall this be redirect?
		}
	}

	private String redirectView(String viewid) {
		return REDIRECT + viewid;
	}

	/**
	 * start a subflow from current flow. the subflow bean is resolved using the installed
	 * variable resolver. 
	 * 
	 * @param subFlowName the name that the flow bean is registered with in the Jacn config file
	 * @param arg
	 * @return
	 */
	public String startFlow(String subFlowName, Object arg) {
		FacesContext fc = FacesContext.getCurrentInstance();
		synchronized (startingFlowName) {
			this.startingFlowName = subFlowName;
		}
		FlowController subFlow = (FlowController) fc.getApplication()
				.getVariableResolver().resolveVariable(fc, subFlowName);
		if (log.isDebugEnabled()) {
			log.debug("start flow: " + subFlowName + "(" + arg + ")");
		}
		if (subFlow == null) {
			return returnWithMessage(fc, "no flow is defined in the name of: "
					+ subFlowName);
		}

		Flow f = new Flow(subFlow);
		// make a local flow id
		String flowid = getNewFlowId(subFlowName);
		f.setFlowid(flowid);
		f.setControllerName(subFlowName); // shall this be the flow id?
		flowStack.pushFlow(f);
		return REDIRECT + subFlow.begin(arg);
	}

	/**
	 * start a flow of the flowControllerClass type, assuming there is only one instance of
	 * which is being deployed in the bean container. Usually this is a more convenient method
	 * since the caller does not need to know the name under which the bean has been deployed 
	 * in the container. 
	 * @param flowControllerClass
	 * @param arg
	 * @return
	 */
	public String startFlow(Class flowControllerClass, Object arg) {
		if (!FlowController.class.isAssignableFrom(flowControllerClass)) {
			throw new RuntimeException("cannot start flow with a none-FlowController: " +
					flowControllerClass.getName());
		}
		
		FacesContext fc = FacesContext.getCurrentInstance();
		JacnWebApplicationContext jacnCtx = (JacnWebApplicationContext) FacesContextUtils
			.getRequiredWebApplicationContext(fc);
		
		String[] beanNames = jacnCtx.getBeanNamesForType(flowControllerClass);
		String subFlowBeanName = "";
		if (beanNames.length == 0) {
			String msg = "the flow bean is not deployed in Jacn config file:" + flowControllerClass;
			log.error(msg);
			throw new RuntimeException(msg);
		} else if (beanNames.length > 1) {
			log.warn("found multiple deployments of " + flowControllerClass.getName() + ". Use the first one: " + subFlowBeanName);
		}
		subFlowBeanName = beanNames[0];
		
		// naming pattern coupling
		// use the proxy if the target is proxied.
		//
		if (subFlowBeanName.endsWith(JacnClassVisitor.PROXY_TARGET_APPENDIX)) {
			subFlowBeanName = subFlowBeanName.substring(0, subFlowBeanName.length() 
					- JacnClassVisitor.PROXY_TARGET_APPENDIX.length());
		}
		
		return startFlow(subFlowBeanName, arg);
		
	}

	private String returnWithMessage(FacesContext fc, String msg) {
		FacesMessage facesMessage = new FacesMessage(msg);
		fc.addMessage(null, facesMessage);
		return resumeCurrent(null);
	}
	
	/**
	 * return the bean in the current flow
	 * 
	 * @param name
	 */
	public Object getBean(String name) {
		Object bean = null;
		Flow flow = flowStack.peekFlow();
		if (flow != null) {
			bean = flow.getBean(name);
		}
		return bean;
	}

	/**
	 * return the bean in the current flow or ancestors
	 * 
	 * @param name
	 */
	public Object getBeanRecursive(String name) {
		return flowStack.getBeanRecursive(name);
//		Object bean = null;
//		Flow flow = flowStack.peekFlow();
//		if (flow != null) {
//			bean = flow.getBean(name);
//			if (bean == null ) {
//				// search parent 
//				String fid = flow.getFlowid();
//			}
//		}
//		return bean;
	}
	
	public void setBean(String name, Object bean) {
		synchronized (this.startingFlowName) {
			if (name.equals(startingFlowName + JacnClassVisitor.PROXY_TARGET_APPENDIX)) {
				// don'r store the proxied flow controller target when is starting the flow
				return;
			}
		}
		Flow flow = flowStack.peekFlow();
		if (flow != null) {
			flow.putBean(name, bean);
		}
		else {
			String fid = getContextFlowPath();
			throw new RuntimeException ("the flowstacck is empty while the context contains flowid: " + fid);
		}
	}

	public void setHomeViewId(String string) {
		this.homeViewId = string;
	}

	public String popViewHint() {
		String hint = this.viewHint;
		this.viewHint = null;
		return hint;
	}

	public static FlowManager getCurrentInstance() {
		return FlowManager
				.getCurrentInstance(FacesContext.getCurrentInstance());
	}

	public String getHomeViewId() {
		return homeViewId;
	}

	/**
	 * link a GET request caused by the redirect to the the current flow
	 * 
	 * @param now
	 * @param redirectPath
	 */
	public void linkToFlow(String redirectPath) {
		RedirectEvent redirectEvent = new RedirectEvent(new Date(),
				redirectPath, getContextFlowPath());
		List thingsToRemove = new ArrayList();
		
		for (Iterator iter = redirectList.iterator(); iter.hasNext();) {
			RedirectEvent redi = (RedirectEvent) iter.next();
			if (redi.path.equals(redirectPath)) {
				thingsToRemove.add(redi);
				//redirectList.remove(redi);
			}
		}
		
		redirectList.removeAll(thingsToRemove);
		
		redirectList.add(redirectEvent);
	}

	static class RedirectEvent implements Serializable {
		/**
		 * 
		 */
		private static final long serialVersionUID = 6111947526021055504L;

		Date time;

		String path;

		String flowPath;

		public RedirectEvent(Date time, String redirectPath, String flowPath) {
			this.time = time;
			this.path = redirectPath;
			this.flowPath = flowPath;
		}
	}

	/**
	 * try to match a request path to a redirect event and return the previous
	 * flow id that got lost in the even of redirect.
	 * 
	 * @param requestPath
	 * @return null if no match
	 */
	public String getFlowPathForRedirect(String requestPath) {
		String flowPath = null;
		for (Iterator iter = redirectList.iterator(); iter.hasNext();) {
			RedirectEvent r = (RedirectEvent) iter.next();
			String rpath = r.path;
			if (rpath.equals(requestPath)) {
				boolean isCloseEnough = new Date().getTime() - r.time.getTime() < REQUEST_TTL;
				if (true || isCloseEnough) {
					flowPath = r.flowPath;
					Flow f = flowStack.getFlow(flowPath); 
					if (f == null) {
						redirectList.remove(r);
						f = getValidParentRecursive(flowPath);
						if (f == null) {
							// shoudl really redirect to a global home page
							log.warn("there is no flow to match the requestPath: " + requestPath + ". redirect to home page");
							redirectImmediate(this.getHomeViewId());
						}
						else {
							// redirect to the valid home page
							log.warn("there is no flow to match the requestPath: " + requestPath + ". redirect to parent flow view");
							String vid = f.getFlowController().getHomeView();
							redirectImmediate(vid);
						}
					}
					break;
				}
			}
		}
		return flowPath;
	}

	private void redirectImmediate(String vid) {
		FacesContext fc = FacesContext.getCurrentInstance();
		ExternalContext ec = fc.getExternalContext();
		String redirectPath = fc.getApplication().
			getViewHandler().getActionURL(fc, vid);
		
		try {
			ec.redirect(ec.encodeActionURL(redirectPath));
		} catch (IOException e) {
			throw new FacesException(e.getMessage(), e);
		}
		fc.responseComplete();
	}

	private Flow getValidParentRecursive(String flowPath) {
		Flow f = this.flowStack.getFlow(flowPath);
		if (f == null) {
			// the flow does not exist anymore
			// redirectList.remove(r);
			// go upper flow
			flowPath = FlowTree.getParentPath(flowPath);
			if (flowPath == null || flowPath.length() == 0) {
				return null;
			}
			else {
				f = getValidParentRecursive(flowPath);
			}
		}
		return f;
	}

	public void synchIdWithFlow(String flowPath) {
		Flow f = flowStack.getFlow(flowPath); 
		if (f == null) {
			f = getValidParentRecursive(flowPath);
			if (f == null) {
				// shoudl really redirect to a global home page
				log.warn("there is no flow/parent flow to match the flowPath: " + flowPath
						+ ". redirect to home page");
				redirectImmediate(this.getHomeViewId());
			}
			else {
				// redirect to the valid home page
				log.warn("there is no flow to match the exact path: " + flowPath + ". redirect to ancestor flow view");
				String vid = f.getFlowController().getHomeView();
				if (vid == null || vid.length() == 0 || !vid.startsWith("/")) {
					String err = "bad home view for " + f.getFlowController().getClass().getName() + ". please override getHomeView()";
					log.error(err);
					throw new RuntimeException(err);
				}
				redirectImmediate(vid);
			}
		}
	}
}
