package bran.spring.jacn.faces;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @author bran
 */
public class FlowTree implements Serializable {
	private static final long serialVersionUID = 6384147535384804420L;

	private static final Log log = LogFactory.getLog(FlowTree.class);

	Map pathFlowMap = new HashMap(); // [flowpath, Flow]

	public FlowTree() {
	}

	/**
	 * store the flow object and update the current flow path in the faces
	 * request
	 * 
	 * @param f
	 */
	public synchronized void pushFlow(Flow f) {
		log.debug("flow pushed on stack: " + f.getFlowid());
		FlowManager fm = FlowManager.getCurrentInstance();
		String flowPath = fm.getContextFlowPath();
		flowPath = flowPath == null ? "" : flowPath;
		String newFlowPath = flowPath + "/" + f.getFlowid();
		this.pathFlowMap.put(newFlowPath, f);
		fm.setContextFlowId(newFlowPath);

		if (log.isDebugEnabled()) {
			log.debug("flows on stack after push");
			for (Iterator iter = pathFlowMap.keySet().iterator(); iter
					.hasNext();) {
				String p = (String) iter.next();
				log.debug(p);
			}
		}
	}

	/**
	 * 
	 * pop the current flow from the flow path. The new flowPath is set in the
	 * context. This is considered multi-threaded flow safe.
	 * 
	 * @return
	 */
	public synchronized Flow popFlow() {
		FlowManager fm = FlowManager.getCurrentInstance();
		String fpath = fm.getContextFlowPath();
		if (fpath != null && fpath.length() > 0) {
			Flow flow = (Flow) pathFlowMap.get(fpath);
			pathFlowMap.remove(fpath);
			log.debug("flow popped off stack: " + flow.getFlowid());

			fpath = getParentPath(fpath);
			fm.setContextFlowId(fpath);

			// if (fpath == null || fpath.length() == 0) {
			// // if (pathFlowMap.size() != 0) {
			// // log.error("there still something on the pathFlowMap while
			// // the")
			// // }
			// } else {
			// // not really doing anything except some synch check
			// Flow pf = (Flow) pathFlowMap.get(fpath);
			// if (pf == null) {
			// String m = "there nothing left on the pathFlowMap for flow: "
			// + fpath;
			// log.error(m);
			// // TODO: synch
			// throw new RuntimeException(m);
			// }
			//
			// if (log.isDebugEnabled()) {
			// log.debug("flows on stack after pop");
			// for (Iterator iter = pathFlowMap.keySet().iterator(); iter
			// .hasNext();) {
			// String p = (String) iter.next();
			// log.debug(p);
			// }
			// }
			// }

			return flow;
		} else {
			log.debug("could not get flow id from request map");
		}
		return null;
	}

	public static String getParentPath(String fpath) {
		String pp = "";

		if (fpath == null) {
			return "";
		}

		int sepp = fpath.lastIndexOf("/");
		if (sepp >= 0) {
			pp = fpath.substring(0, sepp);
		}
		return pp;
	}

	/**
	 * return the current Flow object
	 * 
	 * @return
	 */
	public Flow peekFlow() {
		String fid = FlowManager.getCurrentInstance().getContextFlowPath();
		return (Flow) pathFlowMap.get(fid);
	}

	public Flow getFlow(String flowPath) {
		return (Flow) pathFlowMap.get(flowPath);
	}

	/**
	 * search the current flow and all ancestors for the first bean matching the
	 * bean name
	 * 
	 * @param name
	 *            the bean name
	 * @return
	 */
	public Object getBeanRecursive(String name) {
		String fid = FlowManager.getCurrentInstance().getContextFlowPath();
		return getBeanRecursive2(name, fid);
	}

	/**
	 * search the flow with the id and all ancestors for the first bean matching
	 * the bean name
	 * 
	 * @param name
	 *            the bean name
	 * @return
	 */
	public Object getBeanRecursive2(String name, String fid) {
		if (fid == null || fid.length() == 0) {
			return null;
		}

		Flow flow = (Flow) pathFlowMap.get(fid);
		Object bean = null;
		if (flow == null) {
			String s = "now flow is associated with the flow id: " + fid;
			log.error(s);
			throw new RuntimeException(s);
		} else {
			bean = flow.getBean(name);
			if (bean == null) {
				fid = getParentPath(fid);
				return getBeanRecursive2(name, fid);
			} else {
				return bean;
			}
		}
	}

	/**
	 * pop the flow and all the child flows of the specified flow controller
	 * Note: it may not actaully pop the flow, if there is branches from the
	 * node All the none-branched nodes are removed from the target flow till
	 * the current flow path.
	 * 
	 * @param flowCon
	 * @return
	 */
	public Flow popFlowUpto(FlowController flowCon) {
		// TODO: to be unit tested
		// if (currentFlowPath != null && currentFlowPath.length() > 0) {

		String uptoPath = null;
		Flow targetFlow = null;
		for (Iterator iter = pathFlowMap.keySet().iterator(); iter.hasNext();) {
			String p = (String) iter.next();
			Flow f = (Flow) pathFlowMap.get(p);
			FlowController curFlowController = f.getFlowController();
			if (curFlowController == flowCon) {
				uptoPath = p;
				targetFlow = f;
				break;
			} else { // handle cases where the flow controller is a cglib
						// enhanced version
				if (curFlowController.getClass().getName().startsWith(
						flowCon.getClass().getName())
						&& curFlowController.hashCode() == flowCon.hashCode()) {
					uptoPath = p;
					targetFlow = f;
					break;
				}
			}
		}

		if (uptoPath != null) {
			FlowManager fm = FlowManager.getCurrentInstance();
			String currentFlowPath = fm.getContextFlowPath();
			if (currentFlowPath != null) {
				// start from /p1/p2/p3 to /p1/p2/p3/p4/p5/p6
				// keep /p1/p2/p3/p41/p5, etc
				// prune only the nodes that does not have branches
				List tobePoppedCandidates = new ArrayList();
				for (Iterator iter = pathFlowMap.keySet().iterator(); iter
						.hasNext();) {
					String p = (String) iter.next();
					if (p.startsWith(uptoPath) && currentFlowPath.startsWith(p)) {
						// this is node on the path.
						tobePoppedCandidates.add(p);
					}
				}

				for (Iterator iter = tobePoppedCandidates.iterator(); iter
						.hasNext();) {
					String candidatePath = (String) iter.next();
					if (!hasMultipleChildrenFlows(candidatePath)) {
						pathFlowMap.remove(candidatePath);
						log.debug("flow popped off stack: " + candidatePath);
					}
				}

				String ppath = getParentPath(uptoPath);
				fm.setContextFlowId(ppath);
				log.debug("set current flow path to: " + ppath);
			}
		} else {
			log.error("could not find a flow to match the flowcontroller: "
					+ flowCon.getClass().getName());
		}

		return targetFlow;
	}

	/**
	 * determine if the path has more than one kid
	 * 
	 * @param path
	 * @return
	 */
	boolean hasMultipleChildrenFlows(String path) {
		int kids = 0;
		for (Iterator it = pathFlowMap.keySet().iterator(); it.hasNext();) {
			String pa = (String) it.next();
			if (pa.startsWith(path) && pa.length() > path.length()) {
				kids++;
			}
		}

		return kids > 1 ? true : false;
	}
}
