package bran.spring.jacn.faces;

import java.io.IOException;
import java.util.Date;

import javax.faces.FacesException;
import javax.faces.application.NavigationHandler;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.jsf.FacesContextUtils;

import bran.spring.jacn.JacnWebApplicationContext;

/**
 * modified from MyFaces NavigationHandlerImpl
 * 
 * @author bran
 * 
 */
public class JacnNavigationHandler extends NavigationHandler {
	private static final Log log = LogFactory
			.getLog(JacnNavigationHandler.class);

	private JacnWebApplicationContext wctx;

	private String redirect = FlowManager.REDIRECT;

	public JacnNavigationHandler() {
		if (log.isTraceEnabled())
			log.trace("New NavigationHandler instance created");
	}

	/**
	 * only support the real view id as the outcome. e.g., view:/dir/view.xhtml,
	 * redirect:/dir/view.xhtml
	 */
	public void handleNavigation(FacesContext facesContext, String fromAction,
			String outcome) {
		if (outcome == null || outcome.equals("")) {
			// stay on current ViewRoot
			return;
		}

		String viewid = outcome;
		boolean isRedirect = false;

		
		if (outcome.startsWith("view:")) {
			viewid = outcome.substring("view:".length());
		} else if (outcome.startsWith(redirect)) {
			viewid = outcome.substring(redirect.length());
			isRedirect = true;
		} else {
		}

		if (wctx == null) {
			// I might build something to support externalizing the navigation
			wctx = (JacnWebApplicationContext) FacesContextUtils
					.getRequiredWebApplicationContext(facesContext);
		}

		//String viewId = facesContext.getViewRoot().getViewId();

		ViewHandler viewHandler = facesContext.getApplication()
				.getViewHandler();

		if (isRedirect) {
			// to keep the flow id which does not survive the redirect. i need to tell
			// the flowmananger to expect a quick get response with nothing else and match that
			// with the current flow
			// another option is to throw the flow id in the request query (thus polluting the url). 
			// don't do that for now
			
			ExternalContext ec = facesContext.getExternalContext();
			String redirectPath = viewHandler.getActionURL(facesContext, viewid);

			// save it with flowmanager
			FlowManager.getCurrentInstance().linkToFlow(redirectPath);
			
			try {
				ec.redirect(ec.encodeActionURL(redirectPath));
			} catch (IOException e) {
				throw new FacesException(e.getMessage(), e);
			}
			facesContext.responseComplete();
		} else {
			UIViewRoot viewRoot = viewHandler.createView(facesContext, viewid);
			viewRoot.setViewId(viewid);
			facesContext.setViewRoot(viewRoot);
			facesContext.renderResponse();
		}
	}
}
