package bran.spring.jacn.faces;

import javax.faces.event.PhaseId;

import java.util.Iterator;
import java.util.Map;

import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 */
public class SpringFacesPhaseListener implements PhaseListener {
	private static final Log log = LogFactory
			.getLog(SpringFacesPhaseListener.class);

	public PhaseId getPhaseId() {
		return PhaseId.ANY_PHASE;
	}

	public void beforePhase(PhaseEvent event) {
		if (log.isDebugEnabled()) {
			log.debug("before phase: " + event.getPhaseId());
		}
		FacesContext fc = event.getFacesContext();

		if (event.getPhaseId() == PhaseId.RESTORE_VIEW) {
			ExternalContext ec = fc.getExternalContext();
			String path = ec.getRequestContextPath() + ec.getRequestServletPath();
			if (log.isDebugEnabled()) {
				log.debug("-------- new request : " + path);
			}
		} else if (event.getPhaseId() == PhaseId.INVOKE_APPLICATION) {
		} else if (event.getPhaseId() == PhaseId.UPDATE_MODEL_VALUES) {
		} else if (event.getPhaseId() == PhaseId.PROCESS_VALIDATIONS) {
		} else if (event.getPhaseId() == PhaseId.APPLY_REQUEST_VALUES) {
		} else if (event.getPhaseId() == PhaseId.RENDER_RESPONSE) {
			// set the flow-id
			
			UIViewRoot viewRoot = fc.getViewRoot();
			String fid = FlowManager.getCurrentInstance().getContextFlowPath();
			if (fid != null && fid.trim().length() > 0) {
				log.debug("set flowid on viewroot: " + fid);
				viewRoot.getAttributes().put(FlowManager.FLOW_ID, fid);
			} else {
				viewRoot.getAttributes().remove(FlowManager.FLOW_ID);
			}
		}
	}

	public void afterPhase(PhaseEvent event) {
		log.debug("after phase: " + event.getPhaseId());

		FacesContext fc = event.getFacesContext();
		ExternalContext ec = fc.getExternalContext();
		if (event.getPhaseId() == PhaseId.RESTORE_VIEW) {
			if (fc == null) {
				log.debug("facesContext not create yet.");
			} else {
				UIViewRoot viewRoot = fc.getViewRoot();
				Map attr = viewRoot.getAttributes();
				String fid = (String) attr.get(FlowManager.FLOW_ID);
				if (log.isDebugEnabled()) {
					log.debug("flow id after restore view: " + fid);
				}
				FlowManager fm = FlowManager.getCurrentInstance(fc);

				if (fid == null) {
					// handle redirect setuation
					HttpServletRequest req = (HttpServletRequest) ec
							.getRequest();
					String meth = req.getMethod();
					if ("GET".equals(meth)) {
						String path = ec.getRequestContextPath()
								+ ec.getRequestServletPath();
						if (path != null) {
							fid = fm.getFlowPathForRedirect(path);
						}
					}
				}
				else {
					// verify that we have the flow in the flowmanager
					fm.synchIdWithFlow(fid);
				}
				fm.setContextFlowId(fid);

				// log.info("view id: " + viewId);
			}
		} else if (event.getPhaseId() == PhaseId.INVOKE_APPLICATION) {
		} else if (event.getPhaseId() == PhaseId.UPDATE_MODEL_VALUES) {
		} else if (event.getPhaseId() == PhaseId.PROCESS_VALIDATIONS) {
		} else if (event.getPhaseId() == PhaseId.APPLY_REQUEST_VALUES) {
		} else if (event.getPhaseId() == PhaseId.RENDER_RESPONSE) {
			if (log.isDebugEnabled()) {
				UIViewRoot viewRoot = fc.getViewRoot();
				String fid = (String) fc.getExternalContext().getRequestMap().get(
						FlowManager.FLOW_ID);
				log.debug("flow id in view after rendering: " + fid);
			}

		}
	}

//	private static HttpSession getSession(PhaseEvent event) {
//		return (HttpSession) event.getFacesContext().getExternalContext()
//				.getSession(true);
//	}
//
//	private static Map getAttributes(PhaseEvent event) {
//		return event.getFacesContext().getViewRoot().getAttributes();
//	}

}
