package bran.spring.jacn.faces;

import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.VariableResolver;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.jsf.FacesContextUtils;

import bran.spring.jacn.JacnWebApplicationContext;
import bran.spring.jacn.WebScope;

public class SpringFacesVariableResolver extends VariableResolver {
	// public static final String FLOWID = "_flowid_";

	private static final Log log = LogFactory
			.getLog(SpringFacesVariableResolver.class);

	// private static ThreadLocal resolvedVarMap = new ThreadLocal() {
	// protected synchronized Object initialValue() {
	// // store <beanName, bean>
	// return new HashMap();
	// }
	// };

	JacnWebApplicationContext wctx = null;

	private VariableResolver originalVariableResolver;

	public SpringFacesVariableResolver(VariableResolver originalVariableResolver) {
		super();
		this.originalVariableResolver = originalVariableResolver;
	}

	/**
	 * resolve bean in JacnWebApplicationContext. use request to cache the
	 * result. Inject implicit web objects
	 */
public Object resolveVariable(FacesContext context, String name)
			throws EvaluationException {
		if (log.isDebugEnabled()) {
			log.debug(name);
		}
		if (wctx == null) {
			wctx = (JacnWebApplicationContext) FacesContextUtils
					.getRequiredWebApplicationContext(context);
		}

		Map reqMap = context.getExternalContext().getRequestMap();
		Object bean = reqMap.get(name);
		if (bean != null) {
			// use it
		} else {
			FlowManager fm = FlowManager.getCurrentInstance(context);
			// Flow flow = fm.getFlow(flowid);
			if (fm != null) {
				bean = fm.getBean(name);
			}
			
			if (bean != null){
				reqMap.put(name, bean);
				injectImplicitObject(bean, context);
			}
			else {
				if (wctx.containsBean(name)) {
					if (log.isDebugEnabled()) {
						log.debug("resolve in JacnWebApplicationContext");
					}
					bean = wctx.getBean(name);
					reqMap.put(name, bean);
					injectImplicitObject(bean, context);
				} else {
					String decName = name + "_outjected";
					// for tmp var in a view such as a datagrid

					bean = reqMap.get(decName);
					if (bean != null) {
						return bean;
					} else {
						// let's earch the outjected beans
						bean = fm.getBeanRecursive(name);

						if (bean == null) {
							decName = name + "_tmp"; // for tmp var in a view
														// such as
							// a datagrid
							bean = reqMap.get(decName);
							if (bean == null) {
								if (log.isDebugEnabled()) {
									log
											.debug(name
													+ " not in JacnWebApplicationContext. "
													+ "resolve in originalVariableResolver");
								}
								bean = this.originalVariableResolver
										.resolveVariable(context, name);
								reqMap.put(decName, bean);
								injectImplicitObject(bean, context);
							} else {
								if (log.isDebugEnabled()) {
									log
											.debug("return cached object of unmanaged bean: "
													+ name);
								}
							}
						} else {
							reqMap.put(decName, bean);
							injectImplicitObject(bean, context);
						}
					}
				}
			}
		}

		// if (bean instanceof FlowController) {
		// FlowManager fm = FlowManager.getCurrentInstance(context);
		// String cfid = fm.getContextFlowPath();
		// // if (cfid.equals(fpath)) {
		// //
		// // }
		// // else {
		// // // should go to default page for the exception
		// // //fm.
		// // }
		// }
		// TODO: shall I move this to the else block for newly create bean only?

		return bean;
	}
	/***************************************************************************
	 * inject some of the faces implicit objects
	 * 
	 * @param bean
	 */
	private void injectImplicitObject(Object bean, FacesContext facesContext) {

		if (facesContext != null) {
			ExternalContext ectx = facesContext.getExternalContext();
			if (bean instanceof FacesContextAware) {
				// log.debug("inject FacesContext");
				((FacesContextAware) bean).setFacesContext(facesContext);
			}

			if (bean instanceof RequestParametersAware) {
				// log.debug("inject RequestParameters");
				((RequestParametersAware) bean).setRequestParameters(ectx
						.getRequestParameterMap());
			}

			if (bean instanceof RequestScopeAware) {
				// log.debug("inject RequestScope");
				((RequestScopeAware) bean)
						.setRequestScope(ectx.getRequestMap());
			}

			if (bean instanceof SessionScopeAware) {
				// log.debug("inject SessionScope");
				((SessionScopeAware) bean)
						.setSessionScope(ectx.getSessionMap());
			}

			if (bean instanceof HttpSessionAware) {
				// log.debug("inject HttpSession");
				((HttpSessionAware) bean)
						.setHttpSession(((HttpSession) facesContext
								.getExternalContext().getSession(false)));
			}

			if (bean instanceof FlowController) {
				((FlowController) bean).setFlowManager(FlowManager
						.getCurrentInstance(facesContext));
			}
			if (bean instanceof FlowManagerAware) {
				((FlowManagerAware) bean).setFlowManager(FlowManager
						.getCurrentInstance(facesContext));
			}
		}
	}
}