package bran.jacn.test;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;
import org.springframework.transaction.interceptor.TransactionProxyFactoryBean;

import junit.framework.TestCase;
import bran.spring.jacn.JacnApplicationContext;

public abstract class JacnTestCase extends TestCase {
	private JacnApplicationContext ctx;

	protected void setUp() throws Exception {
		if (getConfigClasses() != null) {
			ctx = new JacnApplicationContext(getConfigClasses(), false);
			// this is to print the generated DOM in XML
			ctx.setPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					System.out.println(evt.getNewValue());
				}
			});
			// build the beanfactory
			ctx.refresh();
		}
	}

	/**
	 * set the config classes for use in the testcase
	 * @return
	 */
	abstract protected Class[] getConfigClasses();

	public JacnApplicationContext getCtx() {
		return ctx;
	}
	
	public static Element getTopBean(Document doc, String id, String className) throws JDOMException {
		Element e = (Element) XPath.selectSingleNode(doc, 
				"/beans/bean[@id='" + id + "' and @class='" + className + "']");
		return e;
	}
	
	public static Element[] getTopBean(Document doc, Class clz) throws JDOMException {
		List l = XPath.selectNodes(doc, "/beans/bean[@class='" + clz.getName() + "']");
		if (l.size() == 0) {
			return new Element[0];
		}
		else {
			Element[] res = new Element[l.size()];
			l.toArray(res);
			return res;
		}
	}

	public static Element getProperty(Element bean, String propName) throws JDOMException {
		Element e = (Element) XPath.selectSingleNode(bean, 
				"property[@name='" + propName + "']");
		return e;
	}

	public static Element getChild(Element ele, String path) throws JDOMException {
		return (Element) XPath.selectSingleNode(ele, path);
	}
	
}
