package bran.jacn.test;

import java.io.IOException;
import java.lang.reflect.Method;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.aop.AfterReturningAdvice;
import org.springframework.aop.MethodBeforeAdvice;
import org.springframework.aop.support.RegexpMethodPointcutAdvisor;

import bran.spring.jacn.AbstractJacnConfigFile;
import bran.spring.jacn.ConvertUtil;
import bran.spring.jacn.JacnApplicationContext;

import junit.framework.TestCase;

public class TestAop extends TestCase {
	static class UserSecurityAdvice implements MethodBeforeAdvice {

		public void before(Method arg0, Object[] arg1, Object arg2)
				throws Throwable {
			System.out.println("before method: " + arg0);
		}
	}

	static class AuditingAdvice implements AfterReturningAdvice {

		public void afterReturning(Object res, Method meth, Object[] args, Object target) throws Throwable {
			System.out.println("after method: " + res);
		}
	}
	
	static class LoggingAround implements MethodInterceptor {

		public Object invoke(MethodInvocation methodInvocation)
				throws Throwable {
			logEvent(methodInvocation.getMethod().getName(), "Entering call.");
			Object result = methodInvocation.proceed();
			logEvent(methodInvocation.getMethod().getName(), "Leaving call. " + result);
			return result;
		}

		private void logEvent(String name, String string) {
			System.out.println(name + ":" + string);

		}
	}
	
	public static class Foo implements IFoo {
		/* (non-Javadoc)
		 * @see bran.jacn.test.IFoo#echo(java.lang.String)
		 */
		public String echo(String e) {
			return e;
		}
	}
	static class Jacn_aop extends AbstractJacnConfigFile {
		//static RegexpMethodPointcutAdvisor userSecurityInterceptor;
		static LoggingAround la; 
		static UserSecurityAdvice usa;
		Foo foo;
		static AuditingAdvice audit; 
		public void wire() throws Exception {
			weaveAspects(foo, new Object[] {IFoo.class}, new String[] {"echo*", "else*"}, new Object[] {audit, usa, la});
			// does not support calling weaveAspects() on the same object twice yet! 
		}
		
	}
	
	public void testWeave() throws IOException {
		//System.out.println(ConvertUtil.jacn2Xml(Jacn_aop.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_aop.class);
		IFoo foo = (IFoo) ctx.getBean("foo");
		foo.echo("hello");
	}
}
