package bran.jacn.test;

import java.io.IOException;
import java.io.Serializable;
import java.util.Map;
import java.util.Properties;

import org.aopalliance.aop.Advice;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;
import org.springframework.aop.Advisor;
import org.springframework.aop.framework.ProxyFactoryBean;
import org.springframework.beans.factory.config.MethodInvokingFactoryBean;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionException;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.interceptor.TransactionInterceptor;
import org.springframework.transaction.interceptor.TransactionProxyFactoryBean;

import bran.spring.jacn.AbstractJacnConfigFile;
import bran.spring.jacn.ConvertUtil;
import bran.spring.jacn.JacnApplicationContext;
import bran.spring.jacn.WebScope;

public class TestEnableTx extends JacnTestCase {

	/**
	 * test if the right xml is generated
	 * 
	 * @throws Exception
	 */
	public void testTxEnablement() throws Exception {
		Document doc = ConvertUtil.jacn2JDom(Jacn_QuickTx.class);

		System.out.println(ConvertUtil.jacn2Xml(Jacn_QuickTx.class));

		Element b = getTopBean(doc, "userManager2",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(b);
		Element p = getProperty(b, "target");
		assertNotNull(p);
		String targetName = "userManager2__target";
		Element c = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(c);

		c = getChild(b, "//property/ref[@bean='txMan']"); // same as w/o the
															// leading '//'
		assertNotNull(c);

		c = getChild(b, "property[@name='proxyInterfaces']/list");
		assertNotNull(c);
		assertTrue(c.getChildren().size() == 2);
		assertEquals(Serializable.class.getName(), ((Element) c.getChildren()
				.get(0)).getText());

		c = getChild(b, "property/props/prop[@key='change*']");
		assertNotNull(c);

		b = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(b);

		c = getChild(b, "property/ref[@bean=\'myDao']");
		assertNotNull(c);
		c = getChild(b, "property/ref[@bean=\'myDao2']");
		assertNotNull(c);

		b = getTopBean(doc, "txMan", DummyTxMan.class.getName());
		assertNotNull(b);

		b = getTopBean(doc, "ad", DummyAdvisor.class.getName());
		assertNotNull(b);

		b = getTopBean(doc, "myDao", MyDao.class.getName());
		assertNotNull(b);

		b = getTopBean(doc, "myDao2", MyDao.class.getName());
		assertNotNull(b);
	}

	public void testCallOnTarget() throws Exception {
		// System.out.println("testCallOnTarget");
		// System.out.println(ConvertUtil.jacn2Xml(Jacn_QuickTx3.class));
		Document doc = ConvertUtil.jacn2JDom(Jacn_QuickTx3.class);

		Element b = getTopBean(doc, "userManager",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(b);
		Element p = getProperty(b, "target");
		assertNotNull(p);
		String targetName = "userManager__target";
		Element c = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(c);

		c = getChild(b, "//property/ref[@bean='txMan']"); // same as w/o the
															// leading '//'
		assertNotNull(c);

		c = getChild(b, "property/props/prop[@key='change*']");
		assertNotNull(c);

		c = getChild(b,
				"property[@name='preInterceptors']/list/ref[@bean='ad']");
		assertNotNull(c);
		c = getChild(b,
				"property[@name='postInterceptors']/list/ref[@bean='ad']");
		assertNotNull(c);

		b = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(b);

		Element[] bs = getTopBean(doc, MethodInvokingFactoryBean.class);
		assertNotNull(b);

		c = getChild(bs[0],
				"property[@name='targetObject' and @ref='userManager__target']");
		assertNotNull(c);

		c = getChild(bs[0],
				"property[@name='targetMethod' and @value='touchMe']");
		assertNotNull(c);

		c = getChild(bs[0],
				"property[@name='targetMethod' and @value='touchMe']");
		assertNotNull(c);

		c = getChild(bs[0], "property[@name='arguments']/list/value");
		assertNotNull(c);
		assertEquals("ouch!", c.getText());

	}

	public void testEnablementPosition() throws Exception {
		// System.out.println("testEnablementPosition");
		// System.out.println(ConvertUtil.jacn2Xml(Jacn_QuickTx2.class));

		Document doc = ConvertUtil.jacn2JDom(Jacn_QuickTx2.class);

		Element b = getTopBean(doc, "userManager",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(b);
		Element p = getProperty(b, "target");
		assertNotNull(p);
		String targetName = "userManager__target";
		Element c = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(c);

		c = getChild(b, "//property/ref[@bean='txMan']"); // same as w/o the
															// leading '//'
		assertNotNull(c);

		c = getChild(b, "property/props/prop[@key='change*']");
		assertNotNull(c);

		b = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(b);

		c = getChild(b, "property/ref[@bean=\'myDao']");
		assertNotNull(c);
		c = getChild(b, "property/ref[@bean=\'myDao2']");
		assertNotNull(c);
	}

	public void testNullInterceptors() throws Exception {
		// System.out.println("testNullInterceptors");
		// System.out.println(ConvertUtil.jacn2Xml(Jacn_QuickTxNull.class));
		Document doc = ConvertUtil.jacn2JDom(Jacn_QuickTxNull.class);

		Element b = getTopBean(doc, "userManager",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(b);
		Element p = getProperty(b, "target");
		assertNotNull(p);
		String targetName = "userManager__target";
		Element c = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(c);

		c = getChild(b, "//property/ref[@bean='txMan']"); // same as w/o the
															// leading '//'
		assertNotNull(c);

		c = getChild(b, "property/props/prop[@key='change*']");
		assertNotNull(c);

		c = getChild(b,
				"property[@name='preInterceptors']/list/ref[@bean='ad']");
		assertNotNull(c);
		c = getChild(b,
				"property[@name='postInterceptors']/list/ref[@bean='ad']");
		assertNull(c);

		b = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(b);
	}

	// //////////////////////////////////
	static class UserManager implements IUM {
		public String getNameFromId(int id) {
			return "me";
		}

		public void changeName(int id, String name) {
		}

		MyDao myDao;

		MyDao myDao2;

		public MyDao getMyDao() {
			return myDao;
		}

		public void setMyDao(MyDao myDao) {
			this.myDao = myDao;
		}

		public MyDao getMyDao2() {
			return myDao2;
		}

		public void setMyDao2(MyDao myDao2) {
			this.myDao2 = myDao2;
		}

		void touchMe(String touch) {
			this.touch = touch;
		}

		String touch;

		public String getTouch() {
			return touch;
		}

		public void setTouch(String touch) {
			this.touch = touch;
		}

		public UserManager(String touch) {
			this.touch = touch;
		}

		public UserManager() {
			System.out.println("new instance of UserManager created");
		}
	}

	static class MyDao {
	}

	static class DummyTxMan implements PlatformTransactionManager {

		public TransactionStatus getTransaction(TransactionDefinition arg0)
				throws TransactionException {
			return null;
		}

		public void commit(TransactionStatus arg0) throws TransactionException {
		}

		public void rollback(TransactionStatus arg0)
				throws TransactionException {
		}

	}

	static class DummyAdvisor implements Advisor {
		public boolean isPerInstance() {
			return false;
		}

		public Advice getAdvice() {
			return null;
		}
	}

	static class Jacn_QuickTx extends AbstractJacnConfigFile {

		static UserManager userManager2;

		static DummyTxMan txMan;

		static DummyAdvisor ad;

		static MyDao myDao;

		static MyDao myDao2;

		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			pu.put("*", "PROPAGATION_REQUIRED,readOnly");

			userManager2.setMyDao(myDao);
			userManager2.setMyDao2(myDao2);
			enableTx(userManager2, txMan, pu, new Class[] { Serializable.class,
					Map.class }, null, null);
		}
	}

	static class Jacn_QuickTx2 extends AbstractJacnConfigFile {

		static UserManager userManager;

		// static DummyTxMan txMan;
		static DummyAdvisor ad;

		static MyDao myDao;

		static MyDao myDao2;

		/**
		 * setter is called after enableTx()
		 */
		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			pu.put("*", "PROPAGATION_REQUIRED,readOnly");
			enableTx(userManager, Jacn_QuickTx.txMan, pu, null,
					new Object[] { ad }, new Object[] { ad });
			userManager.setMyDao(myDao);
			userManager.setMyDao2(myDao2);
		}
	}

	static class Jacn_QuickTx3 extends AbstractJacnConfigFile {

		static UserManager userManager;

		static DummyTxMan txMan;

		static DummyAdvisor ad;

		/**
		 * setter is called after enableTx()
		 */
		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			enableTx(userManager, txMan, pu, null, new Object[] { ad },
					new Object[] { ad });
			userManager.touchMe("ouch!");
		}
	}

	static class Jacn_QuickTxNull extends AbstractJacnConfigFile {

		static UserManager userManager;

		static DummyTxMan txMan;

		static DummyAdvisor ad;

		/**
		 * setter is called after enableTx()
		 */
		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			enableTx(userManager, txMan, pu, null, new Object[] { ad }, null);
		}
	}

	public void testAutoWireTx() throws Exception {
		// System.out.println("testNullInterceptors");
		System.out.println(ConvertUtil.jacn2Xml(Jacn_QuickTxNull.class));
		Document doc = ConvertUtil.jacn2JDom(Jacn_autoTx.class);

		Element targetBean = getTopBean(doc, "userManager",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(targetBean);
		Element p = getProperty(targetBean, "target");
		assertNotNull(p);
		String targetName = "userManager__target";
		Element target = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(target);

		targetBean = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(targetBean);
		Attribute ta = targetBean.getAttribute("autowire");
		assertNotNull(ta);
		assertEquals("byName", ta.getValue());

		targetBean = getTopBean(doc, "userManager2",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(targetBean);
		p = getProperty(targetBean, "target");
		assertNotNull(p);
		targetName = "userManager2__target";
		target = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(target);

		targetBean = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(targetBean);
		ta = targetBean.getAttribute("autowire");
		assertNotNull(ta);
		assertEquals("byType", ta.getValue());

	}

	static class Jacn_autoTx extends AbstractJacnConfigFile {

		static UserManager userManager;

		static UserManager userManager2;

		static DummyTxMan txMan;

		static DummyAdvisor ad;

		/**
		 * setter is called after enableTx()
		 */
		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			enableTx(userManager, txMan, pu, null, new Object[] { ad }, null);
			autoWireByName(userManager);

			enableTx(userManager2, txMan, pu, null, new Object[] { ad }, null);
			autoWireByType(userManager2);
		}
	}

	public void testTxCtorDI() throws Exception {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_TxCtor.class));
		Document doc = ConvertUtil.jacn2JDom(Jacn_TxCtor.class);

		Element targetBean = getTopBean(doc, "userManager",
				TransactionProxyFactoryBean.class.getName());
		assertNotNull(targetBean);
		Element p = getProperty(targetBean, "target");
		assertNotNull(p);
		String targetName = "userManager__target";
		Element target = getChild(p, "ref[@bean='" + targetName + "']");
		assertNotNull(target);

		targetBean = getTopBean(doc, targetName, UserManager.class.getName());
		assertNotNull(targetBean);
		assertEquals("hello", getChild(targetBean, "constructor-arg/value")
				.getText());
		assertEquals("touch", getChild(targetBean,
				"property[@name='touch']/value").getText());
	}

	static class Jacn_TxCtor extends AbstractJacnConfigFile {

		static UserManager userManager;

		static DummyTxMan txMan;

		static DummyAdvisor ad;

		/**
		 * setter is called after enableTx()
		 */
		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			userManager.setTouch("touch");
			enableTx(userManager, txMan, pu, null, new Object[] { ad }, null);
			userManager = new UserManager("hello");
		}
	}

	static class BackingBean {
		IUM userManager;

		public IUM getUserManager() {
			return userManager;
		}

		public void setUserManager(IUM userManager) {
			this.userManager = userManager;
		}

	}

	static class Jacn_useTxBean extends AbstractJacnConfigFile {
		BackingBean bb;

		BackingBean bb2;

		// BackingBean bb3;
		static UserManager userManager;

		static UserManager userManager2;

		static DummyTxMan txMan;

		public void wire() throws Exception {
			Properties pu = new Properties();
			pu.put("change*", "PROPAGATION_REQUIRED");
			enableTx(userManager, txMan, pu, new Class[] { IUM.class }, null,
					null);
			enableTx(userManager2, txMan, pu, null, null, null);
			bb.setUserManager(userManager);
			autoWireByName(bb2);
			// autoWireByType(bb3);
		}
	}

	public void testUseTxBean() throws IOException {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_useTxBean.class));
		JacnApplicationContext ctx = new JacnApplicationContext(
				Jacn_useTxBean.class);

		Object bean = ctx.getBean("userManager");
		assertTrue(bean instanceof IUM);
		assertTrue(!(bean instanceof UserManager));

		bean = ctx.getBean("userManager2");
		assertTrue(bean instanceof UserManager);

		BackingBean bb = (BackingBean) ctx.getBean("bb");
		assertTrue(bb.getUserManager() instanceof IUM);

		bb = (BackingBean) ctx.getBean("bb2");
		assertTrue(bb.getUserManager() instanceof IUM);
		// bb = (BackingBean) ctx.getBean("bb3");
		// assertTrue(bb.getUserManager() instanceof UserManager);
	}

	protected Class[] getConfigClasses() {
		return null;
	}

	/***
	 * the proxy should be lazy inited if the target is none-singleton
	 * @throws IOException
	 * @throws JDOMException
	 */
	public void testNoneSingleTon() throws IOException, JDOMException {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_noneSingleton.class));
		Document doc = ConvertUtil.jacn2JDom(Jacn_noneSingleton.class);

		Element proxy = getTopBean(doc, "userManager", ProxyFactoryBean.class.getName());
		assertNotNull(proxy);

		Element targetBean = getTopBean(doc, "userManager__target", UserManager.class.getName());
		assertNotNull(targetBean);
		
		assertEquals("byName", targetBean.getAttributeValue("autowire"));
		Element daoRef = (Element) XPath.newInstance("//bean/property[@name='myDao']/ref").selectSingleNode(targetBean);
		assertNotNull(daoRef);
		
		
		Element[] inter = getTopBean(doc, TransactionInterceptor.class);
		assertNotNull(inter);
		assertTrue(inter.length == 1);
		assertTrue(inter[0].getAttributeValue("id").endsWith("tx_interceptor"));

		
//		assertEquals("true", targetBean.getAttributeValue("lazy-init"));
	}

	static class Jacn_noneSingleton extends AbstractJacnConfigFile {
		// none-singleton
		UserManager userManager;
		static DummyTxMan txMan;
		MyDao myDao;
		MyDao myDao2;

		public void wire() throws Exception {
			Properties pu = new Properties(); {
				pu.put("change*", "PROPAGATION_REQUIRED");
			}
			enableTx(userManager, txMan, pu, new Class[] { IUM.class }, null, null);
//			enableTx(userManager, txMan, pu, null, null, null);
			userManager.setMyDao(myDao);
			autoWireByName(userManager);
		}
		public static void main(String[] a) throws Exception {
			System.out.println(ConvertUtil.jacn2Xml(Jacn_noneSingleton.class));
		}
	}

	static class Jacn_noneSingletonNoInterface extends AbstractJacnConfigFile {
		// none-singleton
		UserManager userManager;
		static DummyTxMan txMan;
		MyDao myDao;
		
		public void wire() throws Exception {
			Properties pu = new Properties(); {
				pu.put("*", "PROPAGATION_REQUIRED");
			}
			enableTx(userManager, txMan, pu, null, null, null);
			userManager.setMyDao(myDao);
		}
		public static void main(String[] a) throws Exception {
			System.out.println(ConvertUtil.jacn2Xml(Jacn_noneSingletonNoInterface.class));
		}
	}

	public void testNoneSingletonnoNoInterface() throws IOException, JDOMException {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_noneSingletonNoInterface.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_noneSingletonNoInterface.class);
		UserManager um = (UserManager) ctx.getBean("userManager");
		assertTrue(um.getMyDao() != null);
		
	}

	public void testNoneSingleton2() throws IOException, JDOMException {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_noneSingleton.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_noneSingleton.class);
		IUM um1 = (IUM) ctx.getBean("userManager");
		IUM um2 = (IUM) ctx.getBean("userManager");
//		Object um1 = ctx.getBean("userManager");
//		Object um2 = ctx.getBean("userManager");
		assertTrue(um1 != um2);  
	}

	public void testNoneSingletonScope() throws IOException, JDOMException {
//		System.out.println(ConvertUtil.jacn2Xml(Jacn_noneSingleton.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_noneSingleton.class);
		assertEquals(WebScope.NONE_SCOPE, ctx.getScope("userManager"));
		assertEquals(WebScope.APPLICATION_SCOPE, ctx.getScope("txMan"));
		
	}
}
