/*
 * $Log$
 * Revision 1.2  2005/08/20 09:05:24  branaway
 * fixed issues when debug is off
 *
 * Revision 1.1  2005/08/03 12:42:50  branaway
 * added support for web application context integration: JacnWebApplicationContext
 *
 */
package bran.jacn.test;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import junit.framework.TestCase;

import org.springframework.mock.web.MockServletContext;
import org.springframework.web.context.ConfigurableWebApplicationContext;
import org.springframework.web.context.ContextLoader;
import org.springframework.web.context.ContextLoaderListener;
import org.springframework.web.context.WebApplicationContext;

import bran.spring.jacn.JacnWebApplicationContext;

public class TestJacnWebAppCtx extends TestCase {
	
	protected void setUp() throws Exception {
	}
	
	/**
	 * note the app context is loaded from an XML file and a Jacn file. 
	 *
	 */
	public void testMixXmlJacn() {
		ConfigurableWebApplicationContext wac;
		wac = new JacnWebApplicationContext();
		wac.setServletContext(new MockServletContext(""));
		wac.setConfigLocations(new String[] {
				"/bran/jacn/test/ApplicationContext-Foo.xml",
				"classpath*:bran/jacn/test/ApplicationContext_bar.class"
				});
		wac.refresh();
		Foo f = (Foo) wac.getBean("foo");
		assertEquals(f.getBar().getMessage(), Foo.FOO);
		
		wac.setConfigLocations(new String[] {
				"/bran/jacn/test/ApplicationContext-Bar.xml",
				"classpath*:bran/jacn/test/ApplicationContext_foo2.class"
		});
		
		wac.refresh();
		//
		f = (Foo) wac.getBean("foo");
		assertEquals(f.getBar().getMessage(), "hello bar!");
	}
	
	public void testContextLoaderIntegration() {
		MockServletContext sc = new MockServletContext("");
		sc.addInitParameter(ContextLoader.CONFIG_LOCATION_PARAM,
				"classpath*:bran/jacn/test/ApplicationContext_foo2.class " + // note the space
				"/bran/jacn/test/ApplicationContext-Bar.xml");
		
		// set the web app context to use
		sc.addInitParameter(ContextLoader.CONTEXT_CLASS_PARAM, 
				JacnWebApplicationContext.class.getName());
		
		ServletContextListener listener = new ContextLoaderListener();
		ServletContextEvent event = new ServletContextEvent(sc);
		listener.contextInitialized(event);
		
		WebApplicationContext context = (WebApplicationContext)
				sc.getAttribute(WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE);
		assertTrue("Correct WebApplicationContext exposed in ServletContext",
				context instanceof JacnWebApplicationContext);

		Foo f = (Foo) context.getBean("foo");
		assertEquals(f.getBar().getMessage(), "hello bar!");
		
		listener.contextDestroyed(event);
	}
	
	public void testContextLoaderIntegration2() {
		// use jacn config only
		MockServletContext sc = new MockServletContext("");
		sc.addInitParameter(ContextLoader.CONFIG_LOCATION_PARAM,
				"classpath*:bran/jacn/test/ApplicationContext_foo2.class " + // note the space
				"classpath*:bran/jacn/test/ApplicationContext_bar.class ");
		
		// set the web app context to use
		sc.addInitParameter(ContextLoader.CONTEXT_CLASS_PARAM, 
				JacnWebApplicationContext.class.getName());
		
		ServletContextListener listener = new ContextLoaderListener();
		ServletContextEvent event = new ServletContextEvent(sc);
		listener.contextInitialized(event);
		
		WebApplicationContext context = (WebApplicationContext)
				sc.getAttribute(WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE);
		assertTrue("Correct WebApplicationContext exposed in ServletContext",
				context instanceof JacnWebApplicationContext);

		Foo f = (Foo) context.getBean("foo");
		assertEquals(f.getBar().getMessage(), Foo.FOO);
		
		listener.contextDestroyed(event);
	}
}
