package bran.jacn.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import junit.framework.TestCase;

import org.springframework.beans.factory.config.ListFactoryBean;

import bran.spring.jacn.AbstractJacnConfigFile;
import bran.spring.jacn.ConvertUtil;
import bran.spring.jacn.JacnApplicationContext;

public class TestList extends TestCase {

	public void testListFactory() throws Exception {
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_List.class);
		BB b= (BB) ctx.getBean("b");
		assertTrue(b.getListProps().size() == 1);;
		assertEquals(b.getListProps().get(0), "v1");
	}

	public void testAsList() throws Exception {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_List.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_List.class);
		BB b= (BB) ctx.getBean("b1");
		assertTrue(b.getListProps().size() == 1);;
		assertEquals(b.getListProps().get(0), "a1");
	}
	
	public void testListCast() throws Exception {
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_List.class);
		BB b= (BB) ctx.getBean("b2");
		assertTrue(b.getListProps().size() == 3);;
		assertEquals(b.getListProps().get(0), "a1");
	}
	
	static class Jacn_List extends AbstractJacnConfigFile {
		static BB b;
		static BB b1;
		BB b2;
	
		public void wire() throws Exception {
			List list = new ArrayList();
			list.add("v1");
			
			ListFactoryBean lfb = new ListFactoryBean();
			lfb.setSourceList(list);
			//lfb.setTargetListClass(ArrayList.class);  // .class not supported!
			b.setListProps((List) lfb.getObject());   // the pattern of factoryBean usage
			
			// asList usage
			b1.setListProps(Arrays.asList(new Object[] {"a1"})); // not as efficient

			// static cast
		
			AA a = new AA();
			Object l2 = new Object[] {"a1", b1, a};
			b2.setListProps((List) l2); // efficient
			
		}
	}

	public void testListBean() throws Exception {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_List2.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_List2.class);
		BB b= (BB) ctx.getBean("b");
		BB b1= (BB) ctx.getBean("b1");
		
		assertTrue(b.getListProps().size() == 3);
		assertEquals(b.getListProps().get(0), "a1");
		assertEquals(b.getListProps().get(1), b1);
		assertEquals(b.getListProps().get(2).getClass(), AA.class);
	}

	static class Jacn_List2 extends AbstractJacnConfigFile {
		static BB b;
		static BB b1;
		static ArrayList listBean;
		
		public void wire() throws Exception {
			AA a = new AA();
			Object l2 = new Object[] {"a1", b1, a};
			listBean = new ArrayList((Collection)l2);
			b.setListProps(listBean);
		}
	}

	public void testListBean2() throws Exception {
		System.out.println(ConvertUtil.jacn2Xml(Jacn_List3.class));
		JacnApplicationContext ctx = new JacnApplicationContext(Jacn_List3.class);
		List lb= (List) ctx.getBean("listBean");
		assertEquals(lb.getClass(), ArrayList.class);
	}

	static class Jacn_List3 extends AbstractJacnConfigFile {
		static List listBean;
		public void wire() throws Exception {
			listBean = new ArrayList();
		}
	}
}
