/*
 * $Log$
 * Revision 1.2  2005/08/23 05:08:03  branaway
 * added loadPlaceholderProperties() API to eliminate the use of PropertyPlaceholderFactoryBean
 *
 * Revision 1.1  2005/08/04 17:58:03  branaway
 * works with AppFuse now
 *
 */
package bran.jacn.test;

import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.core.io.Resource;

import bran.spring.jacn.AbstractJacnConfigFile;

/**
 * test how to configure a PropertyPlaceholderConfigurer object in an app context
 * The trick here is the casting. See the comment in the config file. 
 * @author bran
 *
 */
public class TestPropertyPlaceHolderBean extends JacnTestCase {

	/**
	 * load the config class
	 */
	protected Class[] getConfigClasses() {
		return new Class[] {Jacn_propPlaceHolder.class,Jacn_loadProps.class};
	}

	public void testPropertyPlaceHolderBean() throws Exception {
		Foo foo = (Foo) getCtx().getBean("foo");
		//	match the value specified in the database.properties file
		assertEquals(foo.getClassName(), "apple"); 
	}

	public void testPropertyPlaceHolderAPI() throws Exception {
		Foo foo = (Foo) getCtx().getBean("foo2");
		assertEquals(foo.getClassName(), "apple"); 
	}
	
	public static class Jacn_propPlaceHolder extends AbstractJacnConfigFile {

		static PropertyPlaceholderConfigurer propertyConfigurer;
		static Foo foo;
		
		public void wire() throws Exception {
			// the placeholder does not have a Java-friendly API, so I use the magic cast here.
			// note I must use new Object[] instead of String[], or the comppiler won't allow the 
			// casting
			propertyConfigurer.setLocations(
					(Resource[]) new Object[]{"classpath:database.properties"}
					);
			foo.setClassName("${hibernate.connection.username}");
		}
	}
	public static class Jacn_loadProps extends AbstractJacnConfigFile {
		
		static Foo foo2;
		// use API instead of using Spring's PropertyPlaceholderConfigurer
		public void wire() throws Exception {
			loadPlaceholderProperties( new String[]{"classpath:database.properties"});
			foo2.setClassName("${hibernate.connection.username}");
		}
	}
}
